"""
Copyright (c) 2017, Jairus Martin.

Distributed under the terms of the MIT License.

The full license is in the file LICENSE, distributed with this software.

Created on July 8, 2017

@author: jrm
"""
from atom.api import (
    Typed, ForwardTyped, Constant, Unicode, observe, set_default
)

from enaml.core.declarative import d_

from .text_view import TextView, ProxyTextView
from .button import Button, ProxyButton
from .toggle_button import ToggleButton, ProxyToggleButton


class ProxyIcon(ProxyTextView):
    """ The abstract definition of a proxy Icon object.

    """
    #: A reference to the Label declaration.
    declaration = ForwardTyped(lambda: Icon)


class ProxyIconButton(ProxyButton):
    #: A reference to the IconButton declaration.
    declaration = ForwardTyped(lambda: IconButton)


class ProxyIconToggleButton(ProxyToggleButton):
    #: A reference to the IconToggleButton declaration.
    declaration = ForwardTyped(lambda: IconToggleButton)


class Icon(TextView):
    """ A simple control for displaying a Icon.

    """

    #: Icons
    PACK_ENTYPO = ('entypo-add-to-list', 'entypo-add-user', 'entypo-address', 'entypo-adjust', 'entypo-air', 'entypo-aircraft', 'entypo-aircraft-landing', 'entypo-aircraft-take-off', 'entypo-align-bottom', 'entypo-align-horizontal-middle', 'entypo-align-left', 'entypo-align-right', 'entypo-align-top', 'entypo-align-vertical-middle', 'entypo-app-store', 'entypo-archive', 'entypo-area-graph', 'entypo-arrow-bold-down', 'entypo-arrow-bold-left', 'entypo-arrow-bold-right', 'entypo-arrow-bold-up', 'entypo-arrow-down', 'entypo-arrow-left', 'entypo-arrow-long-down', 'entypo-arrow-long-left', 'entypo-arrow-long-right', 'entypo-arrow-long-up', 'entypo-arrow-right', 'entypo-arrow-up', 'entypo-arrow-with-circle-down', 'entypo-arrow-with-circle-left', 'entypo-arrow-with-circle-right', 'entypo-arrow-with-circle-up', 'entypo-attachment', 'entypo-awareness-ribbon', 'entypo-back', 'entypo-back-in-time', 'entypo-baidu', 'entypo-bar-graph', 'entypo-basecamp', 'entypo-battery', 'entypo-beamed-note', 'entypo-behance', 'entypo-bell', 'entypo-blackboard', 'entypo-block', 'entypo-book', 'entypo-bookmark', 'entypo-bookmarks', 'entypo-bowl', 'entypo-box', 'entypo-briefcase', 'entypo-browser', 'entypo-brush', 'entypo-bucket', 'entypo-bug', 'entypo-cake', 'entypo-calculator', 'entypo-calendar', 'entypo-camera', 'entypo-ccw', 'entypo-chat', 'entypo-check', 'entypo-chevron-down', 'entypo-chevron-left', 'entypo-chevron-right', 'entypo-chevron-small-down', 'entypo-chevron-small-left', 'entypo-chevron-small-right', 'entypo-chevron-small-up', 'entypo-chevron-thin-down', 'entypo-chevron-thin-left', 'entypo-chevron-thin-right', 'entypo-chevron-thin-up', 'entypo-chevron-up', 'entypo-chevron-with-circle-down', 'entypo-chevron-with-circle-left', 'entypo-chevron-with-circle-right', 'entypo-chevron-with-circle-up', 'entypo-circle', 'entypo-circle-with-cross', 'entypo-circle-with-minus', 'entypo-circle-with-plus', 'entypo-circular-graph', 'entypo-clapperboard', 'entypo-classic-computer', 'entypo-clipboard', 'entypo-clock', 'entypo-cloud', 'entypo-code', 'entypo-cog', 'entypo-colours', 'entypo-compass', 'entypo-controller-fast-backward', 'entypo-controller-fast-forward', 'entypo-controller-jump-to-start', 'entypo-controller-next', 'entypo-controller-paus', 'entypo-controller-play', 'entypo-controller-record', 'entypo-controller-stop', 'entypo-controller-volume', 'entypo-copy', 'entypo-creative-cloud', 'entypo-creative-commons', 'entypo-creative-commons-attribution', 'entypo-creative-commons-noderivs', 'entypo-creative-commons-noncommercial-eu', 'entypo-creative-commons-noncommercial-us', 'entypo-creative-commons-public-domain', 'entypo-creative-commons-remix', 'entypo-creative-commons-share', 'entypo-creative-commons-sharealike', 'entypo-credit', 'entypo-credit-card', 'entypo-crop', 'entypo-cross', 'entypo-cup', 'entypo-cw', 'entypo-cycle', 'entypo-database', 'entypo-dial-pad', 'entypo-direction', 'entypo-document', 'entypo-document-landscape', 'entypo-documents', 'entypo-dot-single', 'entypo-dots-three-horizontal', 'entypo-dots-three-vertical', 'entypo-dots-two-horizontal', 'entypo-dots-two-vertical', 'entypo-download', 'entypo-dribbble', 'entypo-dribbble-with-circle', 'entypo-drink', 'entypo-drive', 'entypo-drop', 'entypo-dropbox', 'entypo-edit', 'entypo-email', 'entypo-emoji-flirt', 'entypo-emoji-happy', 'entypo-emoji-neutral', 'entypo-emoji-sad', 'entypo-erase', 'entypo-eraser', 'entypo-evernote', 'entypo-export', 'entypo-eye', 'entypo-eye-with-line', 'entypo-facebook', 'entypo-facebook-with-circle', 'entypo-feather', 'entypo-fingerprint', 'entypo-flag', 'entypo-flash', 'entypo-flashlight', 'entypo-flat-brush', 'entypo-flattr', 'entypo-flickr', 'entypo-flickr-with-circle', 'entypo-flow-branch', 'entypo-flow-cascade', 'entypo-flow-line', 'entypo-flow-parallel', 'entypo-flow-tree', 'entypo-flower', 'entypo-folder', 'entypo-folder-images', 'entypo-folder-music', 'entypo-folder-video', 'entypo-forward', 'entypo-foursquare', 'entypo-funnel', 'entypo-game-controller', 'entypo-gauge', 'entypo-github', 'entypo-github-with-circle', 'entypo-globe', 'entypo-google', 'entypo-google-drive', 'entypo-google-hangouts', 'entypo-google-play', 'entypo-google-with-circle', 'entypo-graduation-cap', 'entypo-grid', 'entypo-grooveshark', 'entypo-hair-cross', 'entypo-hand', 'entypo-heart', 'entypo-heart-outlined', 'entypo-help', 'entypo-help-with-circle', 'entypo-home', 'entypo-hour-glass', 'entypo-houzz', 'entypo-icloud', 'entypo-image', 'entypo-image-inverted', 'entypo-images', 'entypo-inbox', 'entypo-infinity', 'entypo-info', 'entypo-info-with-circle', 'entypo-instagram', 'entypo-instagram-with-circle', 'entypo-install', 'entypo-key', 'entypo-keyboard', 'entypo-lab-flask', 'entypo-landline', 'entypo-language', 'entypo-laptop', 'entypo-lastfm', 'entypo-lastfm-with-circle', 'entypo-layers', 'entypo-leaf', 'entypo-level-down', 'entypo-level-up', 'entypo-lifebuoy', 'entypo-light-bulb', 'entypo-light-down', 'entypo-light-up', 'entypo-line-graph', 'entypo-link', 'entypo-linkedin', 'entypo-linkedin-with-circle', 'entypo-list', 'entypo-location', 'entypo-location-pin', 'entypo-lock', 'entypo-lock-open', 'entypo-log-out', 'entypo-login', 'entypo-loop', 'entypo-magnet', 'entypo-magnifying-glass', 'entypo-mail', 'entypo-mail-with-circle', 'entypo-man', 'entypo-map', 'entypo-mask', 'entypo-medal', 'entypo-medium', 'entypo-medium-with-circle', 'entypo-megaphone', 'entypo-menu', 'entypo-merge', 'entypo-message', 'entypo-mic', 'entypo-minus', 'entypo-mixi', 'entypo-mobile', 'entypo-modern-mic', 'entypo-moon', 'entypo-mouse', 'entypo-mouse-pointer', 'entypo-music', 'entypo-network', 'entypo-new', 'entypo-new-message', 'entypo-news', 'entypo-newsletter', 'entypo-note', 'entypo-notification', 'entypo-notifications-off', 'entypo-old-mobile', 'entypo-old-phone', 'entypo-onedrive', 'entypo-open-book', 'entypo-palette', 'entypo-paper-plane', 'entypo-paypal', 'entypo-pencil', 'entypo-phone', 'entypo-picasa', 'entypo-pie-chart', 'entypo-pin', 'entypo-pinterest', 'entypo-pinterest-with-circle', 'entypo-plus', 'entypo-popup', 'entypo-power-plug', 'entypo-price-ribbon', 'entypo-price-tag', 'entypo-print', 'entypo-progress-empty', 'entypo-progress-full', 'entypo-progress-one', 'entypo-progress-two', 'entypo-publish', 'entypo-px', 'entypo-px-with-circle', 'entypo-qq', 'entypo-qq-with-circle', 'entypo-quote', 'entypo-radio', 'entypo-raft', 'entypo-raft-with-circle', 'entypo-rainbow', 'entypo-rdio', 'entypo-rdio-with-circle', 'entypo-remove-user', 'entypo-renren', 'entypo-reply', 'entypo-reply-all', 'entypo-resize-100', 'entypo-resize-full-screen', 'entypo-retweet', 'entypo-rocket', 'entypo-round-brush', 'entypo-rss', 'entypo-ruler', 'entypo-save', 'entypo-scissors', 'entypo-scribd', 'entypo-select-arrows', 'entypo-share', 'entypo-share-alternative', 'entypo-shareable', 'entypo-shield', 'entypo-shop', 'entypo-shopping-bag', 'entypo-shopping-basket', 'entypo-shopping-cart', 'entypo-shuffle', 'entypo-signal', 'entypo-sina-weibo', 'entypo-skype', 'entypo-skype-with-circle', 'entypo-slideshare', 'entypo-smashing', 'entypo-sound', 'entypo-sound-mix', 'entypo-sound-mute', 'entypo-soundcloud', 'entypo-sports-club', 'entypo-spotify', 'entypo-spotify-with-circle', 'entypo-spreadsheet', 'entypo-squared-cross', 'entypo-squared-minus', 'entypo-squared-plus', 'entypo-star', 'entypo-star-outlined', 'entypo-stopwatch', 'entypo-stumbleupon', 'entypo-stumbleupon-with-circle', 'entypo-suitcase', 'entypo-swap', 'entypo-swarm', 'entypo-sweden', 'entypo-switch', 'entypo-tablet', 'entypo-tablet-mobile-combo', 'entypo-tag', 'entypo-text', 'entypo-text-document', 'entypo-text-document-inverted', 'entypo-thermometer', 'entypo-thumbs-down', 'entypo-thumbs-up', 'entypo-thunder-cloud', 'entypo-ticket', 'entypo-time-slot', 'entypo-tools', 'entypo-traffic-cone', 'entypo-trash', 'entypo-tree', 'entypo-triangle-down', 'entypo-triangle-left', 'entypo-triangle-right', 'entypo-triangle-up', 'entypo-tripadvisor', 'entypo-trophy', 'entypo-tumblr', 'entypo-tumblr-with-circle', 'entypo-tv', 'entypo-twitter', 'entypo-twitter-with-circle', 'entypo-typing', 'entypo-uninstall', 'entypo-unread', 'entypo-untag', 'entypo-upload', 'entypo-upload-to-cloud', 'entypo-user', 'entypo-users', 'entypo-v-card', 'entypo-video', 'entypo-video-camera', 'entypo-vimeo', 'entypo-vimeo-with-circle', 'entypo-vine', 'entypo-vine-with-circle', 'entypo-vinyl', 'entypo-vk', 'entypo-vk-alternitive', 'entypo-vk-with-circle', 'entypo-voicemail', 'entypo-wallet', 'entypo-warning', 'entypo-water', 'entypo-windows-store', 'entypo-xing', 'entypo-xing-with-circle', 'entypo-yelp', 'entypo-youko', 'entypo-youko-with-circle', 'entypo-youtube', 'entypo-youtube-with-circle')
    PACK_FONT_AWESOME = ('fa-500px', 'fa-adjust', 'fa-adn', 'fa-align-center', 'fa-align-justify', 'fa-align-left', 'fa-align-right', 'fa-amazon', 'fa-ambulance', 'fa-anchor', 'fa-android', 'fa-angellist', 'fa-angle-double-down', 'fa-angle-double-left', 'fa-angle-double-right', 'fa-angle-double-up', 'fa-angle-down', 'fa-angle-left', 'fa-angle-right', 'fa-angle-up', 'fa-apple', 'fa-archive', 'fa-area-chart', 'fa-arrow-circle-down', 'fa-arrow-circle-left', 'fa-arrow-circle-o-down', 'fa-arrow-circle-o-left', 'fa-arrow-circle-o-right', 'fa-arrow-circle-o-up', 'fa-arrow-circle-right', 'fa-arrow-circle-up', 'fa-arrow-down', 'fa-arrow-left', 'fa-arrow-right', 'fa-arrow-up', 'fa-arrows', 'fa-arrows-alt', 'fa-arrows-h', 'fa-arrows-v', 'fa-asterisk', 'fa-at', 'fa-automobile', 'fa-backward', 'fa-balance-scale', 'fa-ban', 'fa-bank', 'fa-bar-chart', 'fa-bar-chart-o', 'fa-barcode', 'fa-bars', 'fa-battery-0', 'fa-battery-1', 'fa-battery-2', 'fa-battery-3', 'fa-battery-4', 'fa-battery-empty', 'fa-battery-full', 'fa-battery-half', 'fa-battery-quarter', 'fa-battery-three-quarters', 'fa-bed', 'fa-beer', 'fa-behance', 'fa-behance-square', 'fa-bell', 'fa-bell-o', 'fa-bell-slash', 'fa-bell-slash-o', 'fa-bicycle', 'fa-binoculars', 'fa-birthday-cake', 'fa-bitbucket', 'fa-bitbucket-square', 'fa-bitcoin', 'fa-black-tie', 'fa-bluetooth', 'fa-bluetooth-b', 'fa-bold', 'fa-bolt', 'fa-bomb', 'fa-book', 'fa-bookmark', 'fa-bookmark-o', 'fa-briefcase', 'fa-btc', 'fa-bug', 'fa-building', 'fa-building-o', 'fa-bullhorn', 'fa-bullseye', 'fa-bus', 'fa-buysellads', 'fa-cab', 'fa-calculator', 'fa-calendar', 'fa-calendar-check-o', 'fa-calendar-minus-o', 'fa-calendar-o', 'fa-calendar-plus-o', 'fa-calendar-times-o', 'fa-camera', 'fa-camera-retro', 'fa-car', 'fa-caret-down', 'fa-caret-left', 'fa-caret-right', 'fa-caret-square-o-down', 'fa-caret-square-o-left', 'fa-caret-square-o-right', 'fa-caret-square-o-up', 'fa-caret-up', 'fa-cart-arrow-down', 'fa-cart-plus', 'fa-cc', 'fa-cc-amex', 'fa-cc-diners-club', 'fa-cc-discover', 'fa-cc-jcb', 'fa-cc-mastercard', 'fa-cc-paypal', 'fa-cc-stripe', 'fa-cc-visa', 'fa-certificate', 'fa-chain', 'fa-chain-broken', 'fa-check', 'fa-check-circle', 'fa-check-circle-o', 'fa-check-square', 'fa-check-square-o', 'fa-chevron-circle-down', 'fa-chevron-circle-left', 'fa-chevron-circle-right', 'fa-chevron-circle-up', 'fa-chevron-down', 'fa-chevron-left', 'fa-chevron-right', 'fa-chevron-up', 'fa-child', 'fa-chrome', 'fa-circle', 'fa-circle-o', 'fa-circle-o-notch', 'fa-circle-thin', 'fa-clipboard', 'fa-clock-o', 'fa-clone', 'fa-close', 'fa-cloud', 'fa-cloud-download', 'fa-cloud-upload', 'fa-cny', 'fa-code', 'fa-code-fork', 'fa-codepen', 'fa-codiepie', 'fa-coffee', 'fa-cog', 'fa-cogs', 'fa-columns', 'fa-comment', 'fa-comment-o', 'fa-commenting', 'fa-commenting-o', 'fa-comments', 'fa-comments-o', 'fa-compass', 'fa-compress', 'fa-connectdevelop', 'fa-contao', 'fa-copy', 'fa-copyright', 'fa-creative-commons', 'fa-credit-card', 'fa-credit-card-alt', 'fa-crop', 'fa-crosshairs', 'fa-css3', 'fa-cube', 'fa-cubes', 'fa-cut', 'fa-cutlery', 'fa-dashboard', 'fa-dashcube', 'fa-database', 'fa-dedent', 'fa-delicious', 'fa-desktop', 'fa-deviantart', 'fa-diamond', 'fa-digg', 'fa-dollar', 'fa-dot-circle-o', 'fa-download', 'fa-dribbble', 'fa-dropbox', 'fa-drupal', 'fa-edge', 'fa-edit', 'fa-eject', 'fa-ellipsis-h', 'fa-ellipsis-v', 'fa-empire', 'fa-envelope', 'fa-envelope-o', 'fa-envelope-square', 'fa-eraser', 'fa-eur', 'fa-euro', 'fa-exchange', 'fa-exclamation', 'fa-exclamation-circle', 'fa-exclamation-triangle', 'fa-expand', 'fa-expeditedssl', 'fa-external-link', 'fa-external-link-square', 'fa-eye', 'fa-eye-slash', 'fa-eyedropper', 'fa-facebook', 'fa-facebook-f', 'fa-facebook-official', 'fa-facebook-square', 'fa-fast-backward', 'fa-fast-forward', 'fa-fax', 'fa-feed', 'fa-female', 'fa-fighter-jet', 'fa-file', 'fa-file-archive-o', 'fa-file-audio-o', 'fa-file-code-o', 'fa-file-excel-o', 'fa-file-image-o', 'fa-file-movie-o', 'fa-file-o', 'fa-file-pdf-o', 'fa-file-photo-o', 'fa-file-picture-o', 'fa-file-powerpoint-o', 'fa-file-sound-o', 'fa-file-text', 'fa-file-text-o', 'fa-file-video-o', 'fa-file-word-o', 'fa-file-zip-o', 'fa-files-o', 'fa-film', 'fa-filter', 'fa-fire', 'fa-fire-extinguisher', 'fa-firefox', 'fa-flag', 'fa-flag-checkered', 'fa-flag-o', 'fa-flash', 'fa-flask', 'fa-flickr', 'fa-floppy-o', 'fa-folder', 'fa-folder-o', 'fa-folder-open', 'fa-folder-open-o', 'fa-font', 'fa-fonticons', 'fa-fort-awesome', 'fa-forumbee', 'fa-forward', 'fa-foursquare', 'fa-frown-o', 'fa-futbol-o', 'fa-gamepad', 'fa-gavel', 'fa-gbp', 'fa-ge', 'fa-gear', 'fa-gears', 'fa-genderless', 'fa-get-pocket', 'fa-gg', 'fa-gg-circle', 'fa-gift', 'fa-git', 'fa-git-square', 'fa-github', 'fa-github-alt', 'fa-github-square', 'fa-gittip', 'fa-glass', 'fa-globe', 'fa-google', 'fa-google-plus', 'fa-google-plus-square', 'fa-google-wallet', 'fa-graduation-cap', 'fa-gratipay', 'fa-group', 'fa-h-square', 'fa-hacker-news', 'fa-hand-grab-o', 'fa-hand-lizard-o', 'fa-hand-o-down', 'fa-hand-o-left', 'fa-hand-o-right', 'fa-hand-o-up', 'fa-hand-paper-o', 'fa-hand-peace-o', 'fa-hand-pointer-o', 'fa-hand-rock-o', 'fa-hand-scissors-o', 'fa-hand-spock-o', 'fa-hand-stop-o', 'fa-hashtag', 'fa-hdd-o', 'fa-header', 'fa-headphones', 'fa-heart', 'fa-heart-o', 'fa-heartbeat', 'fa-history', 'fa-home', 'fa-hospital-o', 'fa-hotel', 'fa-hourglass', 'fa-hourglass-1', 'fa-hourglass-2', 'fa-hourglass-3', 'fa-hourglass-end', 'fa-hourglass-half', 'fa-hourglass-o', 'fa-hourglass-start', 'fa-houzz', 'fa-html5', 'fa-i-cursor', 'fa-ils', 'fa-image', 'fa-inbox', 'fa-indent', 'fa-industry', 'fa-info', 'fa-info-circle', 'fa-inr', 'fa-instagram', 'fa-institution', 'fa-internet-explorer', 'fa-intersex', 'fa-ioxhost', 'fa-italic', 'fa-joomla', 'fa-jpy', 'fa-jsfiddle', 'fa-key', 'fa-keyboard-o', 'fa-krw', 'fa-language', 'fa-laptop', 'fa-lastfm', 'fa-lastfm-square', 'fa-leaf', 'fa-leanpub', 'fa-legal', 'fa-lemon-o', 'fa-level-down', 'fa-level-up', 'fa-life-bouy', 'fa-life-buoy', 'fa-life-ring', 'fa-life-saver', 'fa-lightbulb-o', 'fa-line-chart', 'fa-link', 'fa-linkedin', 'fa-linkedin-square', 'fa-linux', 'fa-list', 'fa-list-alt', 'fa-list-ol', 'fa-list-ul', 'fa-location-arrow', 'fa-lock', 'fa-long-arrow-down', 'fa-long-arrow-left', 'fa-long-arrow-right', 'fa-long-arrow-up', 'fa-magic', 'fa-magnet', 'fa-mail-forward', 'fa-mail-reply', 'fa-mail-reply-all', 'fa-male', 'fa-map', 'fa-map-marker', 'fa-map-o', 'fa-map-pin', 'fa-map-signs', 'fa-mars', 'fa-mars-double', 'fa-mars-stroke', 'fa-mars-stroke-h', 'fa-mars-stroke-v', 'fa-maxcdn', 'fa-meanpath', 'fa-medium', 'fa-medkit', 'fa-meh-o', 'fa-mercury', 'fa-microphone', 'fa-microphone-slash', 'fa-minus', 'fa-minus-circle', 'fa-minus-square', 'fa-minus-square-o', 'fa-mixcloud', 'fa-mobile', 'fa-mobile-phone', 'fa-modx', 'fa-money', 'fa-moon-o', 'fa-mortar-board', 'fa-motorcycle', 'fa-mouse-pointer', 'fa-music', 'fa-navicon', 'fa-neuter', 'fa-newspaper-o', 'fa-object-group', 'fa-object-ungroup', 'fa-odnoklassniki', 'fa-odnoklassniki-square', 'fa-opencart', 'fa-openid', 'fa-opera', 'fa-optin-monster', 'fa-outdent', 'fa-pagelines', 'fa-paint-brush', 'fa-paper-plane', 'fa-paper-plane-o', 'fa-paperclip', 'fa-paragraph', 'fa-paste', 'fa-pause', 'fa-pause-circle', 'fa-pause-circle-o', 'fa-paw', 'fa-paypal', 'fa-pencil', 'fa-pencil-square', 'fa-pencil-square-o', 'fa-percent', 'fa-phone', 'fa-phone-square', 'fa-photo', 'fa-picture-o', 'fa-pie-chart', 'fa-pied-piper', 'fa-pied-piper-alt', 'fa-pinterest', 'fa-pinterest-p', 'fa-pinterest-square', 'fa-plane', 'fa-play', 'fa-play-circle', 'fa-play-circle-o', 'fa-plug', 'fa-plus', 'fa-plus-circle', 'fa-plus-square', 'fa-plus-square-o', 'fa-power-off', 'fa-print', 'fa-product-hunt', 'fa-puzzle-piece', 'fa-qq', 'fa-qrcode', 'fa-question', 'fa-question-circle', 'fa-quote-left', 'fa-quote-right', 'fa-ra', 'fa-random', 'fa-rebel', 'fa-recycle', 'fa-reddit', 'fa-reddit-alien', 'fa-reddit-square', 'fa-refresh', 'fa-registered', 'fa-remove', 'fa-renren', 'fa-reorder', 'fa-repeat', 'fa-reply', 'fa-reply-all', 'fa-retweet', 'fa-rmb', 'fa-road', 'fa-rocket', 'fa-rotate-left', 'fa-rotate-right', 'fa-rouble', 'fa-rss', 'fa-rss-square', 'fa-rub', 'fa-ruble', 'fa-rupee', 'fa-safari', 'fa-save', 'fa-scissors', 'fa-scribd', 'fa-search', 'fa-search-minus', 'fa-search-plus', 'fa-sellsy', 'fa-send', 'fa-send-o', 'fa-server', 'fa-share', 'fa-share-alt', 'fa-share-alt-square', 'fa-share-square', 'fa-share-square-o', 'fa-shekel', 'fa-sheqel', 'fa-shield', 'fa-ship', 'fa-shirtsinbulk', 'fa-shopping-bag', 'fa-shopping-basket', 'fa-shopping-cart', 'fa-sign-in', 'fa-sign-out', 'fa-signal', 'fa-simplybuilt', 'fa-sitemap', 'fa-skyatlas', 'fa-skype', 'fa-slack', 'fa-sliders', 'fa-slideshare', 'fa-smile-o', 'fa-soccer-ball-o', 'fa-sort', 'fa-sort-alpha-asc', 'fa-sort-alpha-desc', 'fa-sort-amount-asc', 'fa-sort-amount-desc', 'fa-sort-asc', 'fa-sort-desc', 'fa-sort-down', 'fa-sort-numeric-asc', 'fa-sort-numeric-desc', 'fa-sort-up', 'fa-soundcloud', 'fa-space-shuttle', 'fa-spinner', 'fa-spoon', 'fa-spotify', 'fa-square', 'fa-square-o', 'fa-stack-exchange', 'fa-stack-overflow', 'fa-star', 'fa-star-half', 'fa-star-half-empty', 'fa-star-half-full', 'fa-star-half-o', 'fa-star-o', 'fa-steam', 'fa-steam-square', 'fa-step-backward', 'fa-step-forward', 'fa-stethoscope', 'fa-sticky-note', 'fa-sticky-note-o', 'fa-stop', 'fa-stop-circle', 'fa-stop-circle-o', 'fa-street-view', 'fa-strikethrough', 'fa-stumbleupon', 'fa-stumbleupon-circle', 'fa-subscript', 'fa-subway', 'fa-suitcase', 'fa-sun-o', 'fa-superscript', 'fa-support', 'fa-table', 'fa-tablet', 'fa-tachometer', 'fa-tag', 'fa-tags', 'fa-tasks', 'fa-taxi', 'fa-television', 'fa-tencent-weibo', 'fa-terminal', 'fa-text-height', 'fa-text-width', 'fa-th', 'fa-th-large', 'fa-th-list', 'fa-thumb-tack', 'fa-thumbs-down', 'fa-thumbs-o-down', 'fa-thumbs-o-up', 'fa-thumbs-up', 'fa-ticket', 'fa-times', 'fa-times-circle', 'fa-times-circle-o', 'fa-tint', 'fa-toggle-down', 'fa-toggle-left', 'fa-toggle-off', 'fa-toggle-on', 'fa-toggle-right', 'fa-toggle-up', 'fa-trademark', 'fa-train', 'fa-transgender', 'fa-transgender-alt', 'fa-trash', 'fa-trash-o', 'fa-tree', 'fa-trello', 'fa-tripadvisor', 'fa-trophy', 'fa-truck', 'fa-try', 'fa-tty', 'fa-tumblr', 'fa-tumblr-square', 'fa-turkish-lira', 'fa-tv', 'fa-twitch', 'fa-twitter', 'fa-twitter-square', 'fa-umbrella', 'fa-underline', 'fa-undo', 'fa-university', 'fa-unlink', 'fa-unlock', 'fa-unlock-alt', 'fa-unsorted', 'fa-upload', 'fa-usb', 'fa-usd', 'fa-user', 'fa-user-md', 'fa-user-plus', 'fa-user-secret', 'fa-user-times', 'fa-users', 'fa-venus', 'fa-venus-double', 'fa-venus-mars', 'fa-viacoin', 'fa-video-camera', 'fa-vimeo', 'fa-vimeo-square', 'fa-vine', 'fa-vk', 'fa-volume-down', 'fa-volume-off', 'fa-volume-up', 'fa-warning', 'fa-wechat', 'fa-weibo', 'fa-weixin', 'fa-whatsapp', 'fa-wheelchair', 'fa-wifi', 'fa-wikipedia-w', 'fa-windows', 'fa-won', 'fa-wordpress', 'fa-wrench', 'fa-xing', 'fa-xing-square', 'fa-y-combinator', 'fa-y-combinator-square', 'fa-yahoo', 'fa-yc', 'fa-yc-square', 'fa-yelp', 'fa-yen', 'fa-youtube', 'fa-youtube-play', 'fa-youtube-square')
    PACK_IONICONS = ('ion-alert', 'ion-alert-circled', 'ion-android-add', 'ion-android-add-circle', 'ion-android-alarm-clock', 'ion-android-alert', 'ion-android-apps', 'ion-android-archive', 'ion-android-arrow-back', 'ion-android-arrow-down', 'ion-android-arrow-dropdown', 'ion-android-arrow-dropdown-circle', 'ion-android-arrow-dropleft', 'ion-android-arrow-dropleft-circle', 'ion-android-arrow-dropright', 'ion-android-arrow-dropright-circle', 'ion-android-arrow-dropup', 'ion-android-arrow-dropup-circle', 'ion-android-arrow-forward', 'ion-android-arrow-up', 'ion-android-attach', 'ion-android-bar', 'ion-android-bicycle', 'ion-android-boat', 'ion-android-bookmark', 'ion-android-bulb', 'ion-android-bus', 'ion-android-calendar', 'ion-android-call', 'ion-android-camera', 'ion-android-cancel', 'ion-android-car', 'ion-android-cart', 'ion-android-chat', 'ion-android-checkbox', 'ion-android-checkbox-blank', 'ion-android-checkbox-outline', 'ion-android-checkbox-outline-blank', 'ion-android-checkmark-circle', 'ion-android-clipboard', 'ion-android-close', 'ion-android-cloud', 'ion-android-cloud-circle', 'ion-android-cloud-done', 'ion-android-cloud-outline', 'ion-android-color-palette', 'ion-android-compass', 'ion-android-contact', 'ion-android-contacts', 'ion-android-contract', 'ion-android-create', 'ion-android-delete', 'ion-android-desktop', 'ion-android-document', 'ion-android-done', 'ion-android-done-all', 'ion-android-download', 'ion-android-drafts', 'ion-android-exit', 'ion-android-expand', 'ion-android-favorite', 'ion-android-favorite-outline', 'ion-android-film', 'ion-android-folder', 'ion-android-folder-open', 'ion-android-funnel', 'ion-android-globe', 'ion-android-hand', 'ion-android-hangout', 'ion-android-happy', 'ion-android-home', 'ion-android-image', 'ion-android-laptop', 'ion-android-list', 'ion-android-locate', 'ion-android-lock', 'ion-android-mail', 'ion-android-map', 'ion-android-menu', 'ion-android-microphone', 'ion-android-microphone-off', 'ion-android-more-horizontal', 'ion-android-more-vertical', 'ion-android-navigate', 'ion-android-notifications', 'ion-android-notifications-none', 'ion-android-notifications-off', 'ion-android-open', 'ion-android-options', 'ion-android-people', 'ion-android-person', 'ion-android-person-add', 'ion-android-phone-landscape', 'ion-android-phone-portrait', 'ion-android-pin', 'ion-android-plane', 'ion-android-playstore', 'ion-android-print', 'ion-android-radio-button-off', 'ion-android-radio-button-on', 'ion-android-refresh', 'ion-android-remove', 'ion-android-remove-circle', 'ion-android-restaurant', 'ion-android-sad', 'ion-android-search', 'ion-android-send', 'ion-android-settings', 'ion-android-share', 'ion-android-share-alt', 'ion-android-star', 'ion-android-star-half', 'ion-android-star-outline', 'ion-android-stopwatch', 'ion-android-subway', 'ion-android-sunny', 'ion-android-sync', 'ion-android-textsms', 'ion-android-time', 'ion-android-train', 'ion-android-unlock', 'ion-android-upload', 'ion-android-volume-down', 'ion-android-volume-mute', 'ion-android-volume-off', 'ion-android-volume-up', 'ion-android-walk', 'ion-android-warning', 'ion-android-watch', 'ion-android-wifi', 'ion-aperture', 'ion-archive', 'ion-arrow-down-a', 'ion-arrow-down-b', 'ion-arrow-down-c', 'ion-arrow-expand', 'ion-arrow-graph-down-left', 'ion-arrow-graph-down-right', 'ion-arrow-graph-up-left', 'ion-arrow-graph-up-right', 'ion-arrow-left-a', 'ion-arrow-left-b', 'ion-arrow-left-c', 'ion-arrow-move', 'ion-arrow-resize', 'ion-arrow-return-left', 'ion-arrow-return-right', 'ion-arrow-right-a', 'ion-arrow-right-b', 'ion-arrow-right-c', 'ion-arrow-shrink', 'ion-arrow-swap', 'ion-arrow-up-a', 'ion-arrow-up-b', 'ion-arrow-up-c', 'ion-asterisk', 'ion-at', 'ion-backspace', 'ion-backspace-outline', 'ion-bag', 'ion-battery-charging', 'ion-battery-empty', 'ion-battery-full', 'ion-battery-half', 'ion-battery-low', 'ion-beaker', 'ion-beer', 'ion-bluetooth', 'ion-bonfire', 'ion-bookmark', 'ion-bowtie', 'ion-briefcase', 'ion-bug', 'ion-calculator', 'ion-calendar', 'ion-camera', 'ion-card', 'ion-cash', 'ion-chatbox', 'ion-chatbox-working', 'ion-chatboxes', 'ion-chatbubble', 'ion-chatbubble-working', 'ion-chatbubbles', 'ion-checkmark', 'ion-checkmark-circled', 'ion-checkmark-round', 'ion-chevron-down', 'ion-chevron-left', 'ion-chevron-right', 'ion-chevron-up', 'ion-clipboard', 'ion-clock', 'ion-close', 'ion-close-circled', 'ion-close-round', 'ion-closed-captioning', 'ion-cloud', 'ion-code', 'ion-code-download', 'ion-code-working', 'ion-coffee', 'ion-compass', 'ion-compose', 'ion-connection-bars', 'ion-contrast', 'ion-crop', 'ion-cube', 'ion-disc', 'ion-document', 'ion-document-text', 'ion-drag', 'ion-earth', 'ion-easel', 'ion-edit', 'ion-egg', 'ion-eject', 'ion-email', 'ion-email-unread', 'ion-erlenmeyer-flask', 'ion-erlenmeyer-flask-bubbles', 'ion-eye', 'ion-eye-disabled', 'ion-female', 'ion-filing', 'ion-film-marker', 'ion-fireball', 'ion-flag', 'ion-flame', 'ion-flash', 'ion-flash-off', 'ion-folder', 'ion-fork', 'ion-fork-repo', 'ion-forward', 'ion-funnel', 'ion-gear-a', 'ion-gear-b', 'ion-grid', 'ion-hammer', 'ion-happy', 'ion-happy-outline', 'ion-headphone', 'ion-heart', 'ion-heart-broken', 'ion-help', 'ion-help-buoy', 'ion-help-circled', 'ion-home', 'ion-icecream', 'ion-image', 'ion-images', 'ion-information', 'ion-information-circled', 'ion-ionic', 'ion-ios-alarm', 'ion-ios-alarm-outline', 'ion-ios-albums', 'ion-ios-albums-outline', 'ion-ios-americanfootball', 'ion-ios-americanfootball-outline', 'ion-ios-analytics', 'ion-ios-analytics-outline', 'ion-ios-arrow-back', 'ion-ios-arrow-down', 'ion-ios-arrow-forward', 'ion-ios-arrow-left', 'ion-ios-arrow-right', 'ion-ios-arrow-thin-down', 'ion-ios-arrow-thin-left', 'ion-ios-arrow-thin-right', 'ion-ios-arrow-thin-up', 'ion-ios-arrow-up', 'ion-ios-at', 'ion-ios-at-outline', 'ion-ios-barcode', 'ion-ios-barcode-outline', 'ion-ios-baseball', 'ion-ios-baseball-outline', 'ion-ios-basketball', 'ion-ios-basketball-outline', 'ion-ios-bell', 'ion-ios-bell-outline', 'ion-ios-body', 'ion-ios-body-outline', 'ion-ios-bolt', 'ion-ios-bolt-outline', 'ion-ios-book', 'ion-ios-book-outline', 'ion-ios-bookmarks', 'ion-ios-bookmarks-outline', 'ion-ios-box', 'ion-ios-box-outline', 'ion-ios-briefcase', 'ion-ios-briefcase-outline', 'ion-ios-browsers', 'ion-ios-browsers-outline', 'ion-ios-calculator', 'ion-ios-calculator-outline', 'ion-ios-calendar', 'ion-ios-calendar-outline', 'ion-ios-camera', 'ion-ios-camera-outline', 'ion-ios-cart', 'ion-ios-cart-outline', 'ion-ios-chatboxes', 'ion-ios-chatboxes-outline', 'ion-ios-chatbubble', 'ion-ios-chatbubble-outline', 'ion-ios-checkmark', 'ion-ios-checkmark-empty', 'ion-ios-checkmark-outline', 'ion-ios-circle-filled', 'ion-ios-circle-outline', 'ion-ios-clock', 'ion-ios-clock-outline', 'ion-ios-close', 'ion-ios-close-empty', 'ion-ios-close-outline', 'ion-ios-cloud', 'ion-ios-cloud-download', 'ion-ios-cloud-download-outline', 'ion-ios-cloud-outline', 'ion-ios-cloud-upload', 'ion-ios-cloud-upload-outline', 'ion-ios-cloudy', 'ion-ios-cloudy-night', 'ion-ios-cloudy-night-outline', 'ion-ios-cloudy-outline', 'ion-ios-cog', 'ion-ios-cog-outline', 'ion-ios-color-filter', 'ion-ios-color-filter-outline', 'ion-ios-color-wand', 'ion-ios-color-wand-outline', 'ion-ios-compose', 'ion-ios-compose-outline', 'ion-ios-contact', 'ion-ios-contact-outline', 'ion-ios-copy', 'ion-ios-copy-outline', 'ion-ios-crop', 'ion-ios-crop-strong', 'ion-ios-download', 'ion-ios-download-outline', 'ion-ios-drag', 'ion-ios-email', 'ion-ios-email-outline', 'ion-ios-eye', 'ion-ios-eye-outline', 'ion-ios-fastforward', 'ion-ios-fastforward-outline', 'ion-ios-filing', 'ion-ios-filing-outline', 'ion-ios-film', 'ion-ios-film-outline', 'ion-ios-flag', 'ion-ios-flag-outline', 'ion-ios-flame', 'ion-ios-flame-outline', 'ion-ios-flask', 'ion-ios-flask-outline', 'ion-ios-flower', 'ion-ios-flower-outline', 'ion-ios-folder', 'ion-ios-folder-outline', 'ion-ios-football', 'ion-ios-football-outline', 'ion-ios-game-controller-a', 'ion-ios-game-controller-a-outline', 'ion-ios-game-controller-b', 'ion-ios-game-controller-b-outline', 'ion-ios-gear', 'ion-ios-gear-outline', 'ion-ios-glasses', 'ion-ios-glasses-outline', 'ion-ios-grid-view', 'ion-ios-grid-view-outline', 'ion-ios-heart', 'ion-ios-heart-outline', 'ion-ios-help', 'ion-ios-help-empty', 'ion-ios-help-outline', 'ion-ios-home', 'ion-ios-home-outline', 'ion-ios-infinite', 'ion-ios-infinite-outline', 'ion-ios-information', 'ion-ios-information-empty', 'ion-ios-information-outline', 'ion-ios-ionic-outline', 'ion-ios-keypad', 'ion-ios-keypad-outline', 'ion-ios-lightbulb', 'ion-ios-lightbulb-outline', 'ion-ios-list', 'ion-ios-list-outline', 'ion-ios-location', 'ion-ios-location-outline', 'ion-ios-locked', 'ion-ios-locked-outline', 'ion-ios-loop', 'ion-ios-loop-strong', 'ion-ios-medical', 'ion-ios-medical-outline', 'ion-ios-medkit', 'ion-ios-medkit-outline', 'ion-ios-mic', 'ion-ios-mic-off', 'ion-ios-mic-outline', 'ion-ios-minus', 'ion-ios-minus-empty', 'ion-ios-minus-outline', 'ion-ios-monitor', 'ion-ios-monitor-outline', 'ion-ios-moon', 'ion-ios-moon-outline', 'ion-ios-more', 'ion-ios-more-outline', 'ion-ios-musical-note', 'ion-ios-musical-notes', 'ion-ios-navigate', 'ion-ios-navigate-outline', 'ion-ios-nutrition', 'ion-ios-nutrition-outline', 'ion-ios-paper', 'ion-ios-paper-outline', 'ion-ios-paperplane', 'ion-ios-paperplane-outline', 'ion-ios-partlysunny', 'ion-ios-partlysunny-outline', 'ion-ios-pause', 'ion-ios-pause-outline', 'ion-ios-paw', 'ion-ios-paw-outline', 'ion-ios-people', 'ion-ios-people-outline', 'ion-ios-person', 'ion-ios-person-outline', 'ion-ios-personadd', 'ion-ios-personadd-outline', 'ion-ios-photos', 'ion-ios-photos-outline', 'ion-ios-pie', 'ion-ios-pie-outline', 'ion-ios-pint', 'ion-ios-pint-outline', 'ion-ios-play', 'ion-ios-play-outline', 'ion-ios-plus', 'ion-ios-plus-empty', 'ion-ios-plus-outline', 'ion-ios-pricetag', 'ion-ios-pricetag-outline', 'ion-ios-pricetags', 'ion-ios-pricetags-outline', 'ion-ios-printer', 'ion-ios-printer-outline', 'ion-ios-pulse', 'ion-ios-pulse-strong', 'ion-ios-rainy', 'ion-ios-rainy-outline', 'ion-ios-recording', 'ion-ios-recording-outline', 'ion-ios-redo', 'ion-ios-redo-outline', 'ion-ios-refresh', 'ion-ios-refresh-empty', 'ion-ios-refresh-outline', 'ion-ios-reload', 'ion-ios-reverse-camera', 'ion-ios-reverse-camera-outline', 'ion-ios-rewind', 'ion-ios-rewind-outline', 'ion-ios-rose', 'ion-ios-rose-outline', 'ion-ios-search', 'ion-ios-search-strong', 'ion-ios-settings', 'ion-ios-settings-strong', 'ion-ios-shuffle', 'ion-ios-shuffle-strong', 'ion-ios-skipbackward', 'ion-ios-skipbackward-outline', 'ion-ios-skipforward', 'ion-ios-skipforward-outline', 'ion-ios-snowy', 'ion-ios-speedometer', 'ion-ios-speedometer-outline', 'ion-ios-star', 'ion-ios-star-half', 'ion-ios-star-outline', 'ion-ios-stopwatch', 'ion-ios-stopwatch-outline', 'ion-ios-sunny', 'ion-ios-sunny-outline', 'ion-ios-telephone', 'ion-ios-telephone-outline', 'ion-ios-tennisball', 'ion-ios-tennisball-outline', 'ion-ios-thunderstorm', 'ion-ios-thunderstorm-outline', 'ion-ios-time', 'ion-ios-time-outline', 'ion-ios-timer', 'ion-ios-timer-outline', 'ion-ios-toggle', 'ion-ios-toggle-outline', 'ion-ios-trash', 'ion-ios-trash-outline', 'ion-ios-undo', 'ion-ios-undo-outline', 'ion-ios-unlocked', 'ion-ios-unlocked-outline', 'ion-ios-upload', 'ion-ios-upload-outline', 'ion-ios-videocam', 'ion-ios-videocam-outline', 'ion-ios-volume-high', 'ion-ios-volume-low', 'ion-ios-wineglass', 'ion-ios-wineglass-outline', 'ion-ios-world', 'ion-ios-world-outline', 'ion-ipad', 'ion-iphone', 'ion-ipod', 'ion-jet', 'ion-key', 'ion-knife', 'ion-laptop', 'ion-leaf', 'ion-levels', 'ion-lightbulb', 'ion-link', 'ion-load-a', 'ion-load-b', 'ion-load-c', 'ion-load-d', 'ion-location', 'ion-lock-combination', 'ion-locked', 'ion-log-in', 'ion-log-out', 'ion-loop', 'ion-magnet', 'ion-male', 'ion-man', 'ion-map', 'ion-medkit', 'ion-merge', 'ion-mic-a', 'ion-mic-b', 'ion-mic-c', 'ion-minus', 'ion-minus-circled', 'ion-minus-round', 'ion-model-s', 'ion-monitor', 'ion-more', 'ion-mouse', 'ion-music-note', 'ion-navicon', 'ion-navicon-round', 'ion-navigate', 'ion-network', 'ion-no-smoking', 'ion-nuclear', 'ion-outlet', 'ion-paintbrush', 'ion-paintbucket', 'ion-paper-airplane', 'ion-paperclip', 'ion-pause', 'ion-person', 'ion-person-add', 'ion-person-stalker', 'ion-pie-graph', 'ion-pin', 'ion-pinpoint', 'ion-pizza', 'ion-plane', 'ion-planet', 'ion-play', 'ion-playstation', 'ion-plus', 'ion-plus-circled', 'ion-plus-round', 'ion-podium', 'ion-pound', 'ion-power', 'ion-pricetag', 'ion-pricetags', 'ion-printer', 'ion-pull-request', 'ion-qr-scanner', 'ion-quote', 'ion-radio-waves', 'ion-record', 'ion-refresh', 'ion-reply', 'ion-reply-all', 'ion-ribbon-a', 'ion-ribbon-b', 'ion-sad', 'ion-sad-outline', 'ion-scissors', 'ion-search', 'ion-settings', 'ion-share', 'ion-shuffle', 'ion-skip-backward', 'ion-skip-forward', 'ion-social-android', 'ion-social-android-outline', 'ion-social-angular', 'ion-social-angular-outline', 'ion-social-apple', 'ion-social-apple-outline', 'ion-social-bitcoin', 'ion-social-bitcoin-outline', 'ion-social-buffer', 'ion-social-buffer-outline', 'ion-social-chrome', 'ion-social-chrome-outline', 'ion-social-codepen', 'ion-social-codepen-outline', 'ion-social-css3', 'ion-social-css3-outline', 'ion-social-designernews', 'ion-social-designernews-outline', 'ion-social-dribbble', 'ion-social-dribbble-outline', 'ion-social-dropbox', 'ion-social-dropbox-outline', 'ion-social-euro', 'ion-social-euro-outline', 'ion-social-facebook', 'ion-social-facebook-outline', 'ion-social-foursquare', 'ion-social-foursquare-outline', 'ion-social-freebsd-devil', 'ion-social-github', 'ion-social-github-outline', 'ion-social-google', 'ion-social-google-outline', 'ion-social-googleplus', 'ion-social-googleplus-outline', 'ion-social-hackernews', 'ion-social-hackernews-outline', 'ion-social-html5', 'ion-social-html5-outline', 'ion-social-instagram', 'ion-social-instagram-outline', 'ion-social-javascript', 'ion-social-javascript-outline', 'ion-social-linkedin', 'ion-social-linkedin-outline', 'ion-social-markdown', 'ion-social-nodejs', 'ion-social-octocat', 'ion-social-pinterest', 'ion-social-pinterest-outline', 'ion-social-python', 'ion-social-reddit', 'ion-social-reddit-outline', 'ion-social-rss', 'ion-social-rss-outline', 'ion-social-sass', 'ion-social-skype', 'ion-social-skype-outline', 'ion-social-snapchat', 'ion-social-snapchat-outline', 'ion-social-tumblr', 'ion-social-tumblr-outline', 'ion-social-tux', 'ion-social-twitch', 'ion-social-twitch-outline', 'ion-social-twitter', 'ion-social-twitter-outline', 'ion-social-usd', 'ion-social-usd-outline', 'ion-social-vimeo', 'ion-social-vimeo-outline', 'ion-social-whatsapp', 'ion-social-whatsapp-outline', 'ion-social-windows', 'ion-social-windows-outline', 'ion-social-wordpress', 'ion-social-wordpress-outline', 'ion-social-yahoo', 'ion-social-yahoo-outline', 'ion-social-yen', 'ion-social-yen-outline', 'ion-social-youtube', 'ion-social-youtube-outline', 'ion-soup-can', 'ion-soup-can-outline', 'ion-speakerphone', 'ion-speedometer', 'ion-spoon', 'ion-star', 'ion-stats-bars', 'ion-steam', 'ion-stop', 'ion-thermometer', 'ion-thumbsdown', 'ion-thumbsup', 'ion-toggle', 'ion-toggle-filled', 'ion-transgender', 'ion-trash-a', 'ion-trash-b', 'ion-trophy', 'ion-tshirt', 'ion-tshirt-outline', 'ion-umbrella', 'ion-university', 'ion-unlocked', 'ion-upload', 'ion-usb', 'ion-videocamera', 'ion-volume-high', 'ion-volume-low', 'ion-volume-medium', 'ion-volume-mute', 'ion-wand', 'ion-waterdrop', 'ion-wifi', 'ion-wineglass', 'ion-woman', 'ion-wrench', 'ion-xbox')
    PACK_MATERIAL_COMMUNITY = ('mdi-access-point', 'mdi-access-point-network', 'mdi-account', 'mdi-account-alert', 'mdi-account-box', 'mdi-account-box-outline', 'mdi-account-check', 'mdi-account-circle', 'mdi-account-convert', 'mdi-account-key', 'mdi-account-location', 'mdi-account-minus', 'mdi-account-multiple', 'mdi-account-multiple-outline', 'mdi-account-multiple-plus', 'mdi-account-network', 'mdi-account-off', 'mdi-account-outline', 'mdi-account-plus', 'mdi-account-remove', 'mdi-account-search', 'mdi-account-star', 'mdi-account-star-variant', 'mdi-account-switch', 'mdi-adjust', 'mdi-air-conditioner', 'mdi-airballoon', 'mdi-airplane', 'mdi-airplane-off', 'mdi-airplay', 'mdi-alarm', 'mdi-alarm-check', 'mdi-alarm-multiple', 'mdi-alarm-off', 'mdi-alarm-plus', 'mdi-album', 'mdi-alert', 'mdi-alert-box', 'mdi-alert-circle', 'mdi-alert-octagon', 'mdi-alert-outline', 'mdi-alpha', 'mdi-alphabetical', 'mdi-amazon', 'mdi-amazon-clouddrive', 'mdi-ambulance', 'mdi-anchor', 'mdi-android', 'mdi-android-debug-bridge', 'mdi-android-studio', 'mdi-apple', 'mdi-apple-finder', 'mdi-apple-ios', 'mdi-apple-mobileme', 'mdi-apple-safari', 'mdi-appnet', 'mdi-apps', 'mdi-archive', 'mdi-arrange-bring-forward', 'mdi-arrange-bring-to-front', 'mdi-arrange-send-backward', 'mdi-arrange-send-to-back', 'mdi-arrow-all', 'mdi-arrow-bottom-drop-circle', 'mdi-arrow-bottom-left', 'mdi-arrow-bottom-right', 'mdi-arrow-collapse', 'mdi-arrow-down', 'mdi-arrow-down-bold', 'mdi-arrow-down-bold-circle', 'mdi-arrow-down-bold-circle-outline', 'mdi-arrow-down-bold-hexagon-outline', 'mdi-arrow-expand', 'mdi-arrow-left', 'mdi-arrow-left-bold', 'mdi-arrow-left-bold-circle', 'mdi-arrow-left-bold-circle-outline', 'mdi-arrow-left-bold-hexagon-outline', 'mdi-arrow-right', 'mdi-arrow-right-bold', 'mdi-arrow-right-bold-circle', 'mdi-arrow-right-bold-circle-outline', 'mdi-arrow-right-bold-hexagon-outline', 'mdi-arrow-top-left', 'mdi-arrow-top-right', 'mdi-arrow-up', 'mdi-arrow-up-bold', 'mdi-arrow-up-bold-circle', 'mdi-arrow-up-bold-circle-outline', 'mdi-arrow-up-bold-hexagon-outline', 'mdi-assistant', 'mdi-at', 'mdi-attachment', 'mdi-audiobook', 'mdi-auto-fix', 'mdi-auto-upload', 'mdi-autorenew', 'mdi-av-timer', 'mdi-baby', 'mdi-backburger', 'mdi-backspace', 'mdi-backup-restore', 'mdi-bank', 'mdi-barcode', 'mdi-barcode-scan', 'mdi-barley', 'mdi-barrel', 'mdi-basecamp', 'mdi-basket', 'mdi-basket-fill', 'mdi-basket-unfill', 'mdi-battery', 'mdi-battery-10', 'mdi-battery-20', 'mdi-battery-30', 'mdi-battery-40', 'mdi-battery-50', 'mdi-battery-60', 'mdi-battery-70', 'mdi-battery-80', 'mdi-battery-90', 'mdi-battery-alert', 'mdi-battery-charging', 'mdi-battery-charging-100', 'mdi-battery-charging-20', 'mdi-battery-charging-30', 'mdi-battery-charging-40', 'mdi-battery-charging-60', 'mdi-battery-charging-80', 'mdi-battery-charging-90', 'mdi-battery-minus', 'mdi-battery-negative', 'mdi-battery-outline', 'mdi-battery-plus', 'mdi-battery-positive', 'mdi-battery-unknown', 'mdi-beach', 'mdi-beaker', 'mdi-beaker-empty', 'mdi-beaker-empty-outline', 'mdi-beaker-outline', 'mdi-beats', 'mdi-beer', 'mdi-behance', 'mdi-bell', 'mdi-bell-off', 'mdi-bell-outline', 'mdi-bell-plus', 'mdi-bell-ring', 'mdi-bell-ring-outline', 'mdi-bell-sleep', 'mdi-beta', 'mdi-bike', 'mdi-bing', 'mdi-binoculars', 'mdi-bio', 'mdi-biohazard', 'mdi-bitbucket', 'mdi-black-mesa', 'mdi-blackberry', 'mdi-blender', 'mdi-blinds', 'mdi-block-helper', 'mdi-blogger', 'mdi-bluetooth', 'mdi-bluetooth-audio', 'mdi-bluetooth-connect', 'mdi-bluetooth-off', 'mdi-bluetooth-settings', 'mdi-bluetooth-transfer', 'mdi-blur', 'mdi-blur-linear', 'mdi-blur-off', 'mdi-blur-radial', 'mdi-bone', 'mdi-book', 'mdi-book-multiple', 'mdi-book-multiple-variant', 'mdi-book-open', 'mdi-book-open-variant', 'mdi-book-variant', 'mdi-bookmark', 'mdi-bookmark-check', 'mdi-bookmark-music', 'mdi-bookmark-outline', 'mdi-bookmark-outline-plus', 'mdi-bookmark-plus', 'mdi-bookmark-remove', 'mdi-border-all', 'mdi-border-bottom', 'mdi-border-color', 'mdi-border-horizontal', 'mdi-border-inside', 'mdi-border-left', 'mdi-border-none', 'mdi-border-outside', 'mdi-border-right', 'mdi-border-style', 'mdi-border-top', 'mdi-border-vertical', 'mdi-bowling', 'mdi-box', 'mdi-box-cutter', 'mdi-briefcase', 'mdi-briefcase-check', 'mdi-briefcase-download', 'mdi-briefcase-upload', 'mdi-brightness-1', 'mdi-brightness-2', 'mdi-brightness-3', 'mdi-brightness-4', 'mdi-brightness-5', 'mdi-brightness-6', 'mdi-brightness-7', 'mdi-brightness-auto', 'mdi-broom', 'mdi-brush', 'mdi-bug', 'mdi-bulletin-board', 'mdi-bullhorn', 'mdi-bus', 'mdi-cached', 'mdi-cake', 'mdi-cake-layered', 'mdi-cake-variant', 'mdi-calculator', 'mdi-calendar', 'mdi-calendar-blank', 'mdi-calendar-check', 'mdi-calendar-clock', 'mdi-calendar-multiple', 'mdi-calendar-multiple-check', 'mdi-calendar-plus', 'mdi-calendar-remove', 'mdi-calendar-text', 'mdi-calendar-today', 'mdi-call-made', 'mdi-call-merge', 'mdi-call-missed', 'mdi-call-received', 'mdi-call-split', 'mdi-camcorder', 'mdi-camcorder-box', 'mdi-camcorder-box-off', 'mdi-camcorder-off', 'mdi-camera', 'mdi-camera-enhance', 'mdi-camera-front', 'mdi-camera-front-variant', 'mdi-camera-iris', 'mdi-camera-party-mode', 'mdi-camera-rear', 'mdi-camera-rear-variant', 'mdi-camera-switch', 'mdi-camera-timer', 'mdi-candycane', 'mdi-car', 'mdi-car-battery', 'mdi-car-connected', 'mdi-car-wash', 'mdi-carrot', 'mdi-cart', 'mdi-cart-outline', 'mdi-cart-plus', 'mdi-case-sensitive-alt', 'mdi-cash', 'mdi-cash-100', 'mdi-cash-multiple', 'mdi-cash-usd', 'mdi-cast', 'mdi-cast-connected', 'mdi-castle', 'mdi-cat', 'mdi-cellphone', 'mdi-cellphone-android', 'mdi-cellphone-basic', 'mdi-cellphone-dock', 'mdi-cellphone-iphone', 'mdi-cellphone-link', 'mdi-cellphone-link-off', 'mdi-cellphone-settings', 'mdi-certificate', 'mdi-chair-school', 'mdi-chart-arc', 'mdi-chart-areaspline', 'mdi-chart-bar', 'mdi-chart-histogram', 'mdi-chart-line', 'mdi-chart-pie', 'mdi-check', 'mdi-check-all', 'mdi-checkbox-blank', 'mdi-checkbox-blank-circle', 'mdi-checkbox-blank-circle-outline', 'mdi-checkbox-blank-outline', 'mdi-checkbox-marked', 'mdi-checkbox-marked-circle', 'mdi-checkbox-marked-circle-outline', 'mdi-checkbox-marked-outline', 'mdi-checkbox-multiple-blank', 'mdi-checkbox-multiple-blank-outline', 'mdi-checkbox-multiple-marked', 'mdi-checkbox-multiple-marked-outline', 'mdi-checkerboard', 'mdi-chemical-weapon', 'mdi-chevron-double-down', 'mdi-chevron-double-left', 'mdi-chevron-double-right', 'mdi-chevron-double-up', 'mdi-chevron-down', 'mdi-chevron-left', 'mdi-chevron-right', 'mdi-chevron-up', 'mdi-church', 'mdi-cisco-webex', 'mdi-city', 'mdi-clipboard', 'mdi-clipboard-account', 'mdi-clipboard-alert', 'mdi-clipboard-arrow-down', 'mdi-clipboard-arrow-left', 'mdi-clipboard-check', 'mdi-clipboard-outline', 'mdi-clipboard-text', 'mdi-clippy', 'mdi-clock', 'mdi-clock-end', 'mdi-clock-fast', 'mdi-clock-in', 'mdi-clock-out', 'mdi-clock-start', 'mdi-close', 'mdi-close-box', 'mdi-close-box-outline', 'mdi-close-circle', 'mdi-close-circle-outline', 'mdi-close-network', 'mdi-close-octagon', 'mdi-close-octagon-outline', 'mdi-closed-caption', 'mdi-cloud', 'mdi-cloud-check', 'mdi-cloud-circle', 'mdi-cloud-download', 'mdi-cloud-outline', 'mdi-cloud-outline-off', 'mdi-cloud-print', 'mdi-cloud-print-outline', 'mdi-cloud-upload', 'mdi-code-array', 'mdi-code-braces', 'mdi-code-brackets', 'mdi-code-equal', 'mdi-code-greater-than', 'mdi-code-greater-than-or-equal', 'mdi-code-less-than', 'mdi-code-less-than-or-equal', 'mdi-code-not-equal', 'mdi-code-not-equal-variant', 'mdi-code-parentheses', 'mdi-code-string', 'mdi-code-tags', 'mdi-codepen', 'mdi-coffee', 'mdi-coffee-to-go', 'mdi-coin', 'mdi-color-helper', 'mdi-comment', 'mdi-comment-account', 'mdi-comment-account-outline', 'mdi-comment-alert', 'mdi-comment-alert-outline', 'mdi-comment-check', 'mdi-comment-check-outline', 'mdi-comment-multiple-outline', 'mdi-comment-outline', 'mdi-comment-plus-outline', 'mdi-comment-processing', 'mdi-comment-processing-outline', 'mdi-comment-question-outline', 'mdi-comment-remove-outline', 'mdi-comment-text', 'mdi-comment-text-outline', 'mdi-compare', 'mdi-compass', 'mdi-compass-outline', 'mdi-console', 'mdi-contact-mail', 'mdi-content-copy', 'mdi-content-cut', 'mdi-content-duplicate', 'mdi-content-paste', 'mdi-content-save', 'mdi-content-save-all', 'mdi-contrast', 'mdi-contrast-box', 'mdi-contrast-circle', 'mdi-cookie', 'mdi-cow', 'mdi-credit-card', 'mdi-credit-card-multiple', 'mdi-credit-card-scan', 'mdi-crop', 'mdi-crop-free', 'mdi-crop-landscape', 'mdi-crop-portrait', 'mdi-crop-square', 'mdi-crosshairs', 'mdi-crosshairs-gps', 'mdi-crown', 'mdi-cube', 'mdi-cube-outline', 'mdi-cube-send', 'mdi-cube-unfolded', 'mdi-cup', 'mdi-cup-water', 'mdi-currency-btc', 'mdi-currency-eur', 'mdi-currency-gbp', 'mdi-currency-inr', 'mdi-currency-ngn', 'mdi-currency-rub', 'mdi-currency-try', 'mdi-currency-usd', 'mdi-cursor-default', 'mdi-cursor-default-outline', 'mdi-cursor-move', 'mdi-cursor-pointer', 'mdi-database', 'mdi-database-minus', 'mdi-database-plus', 'mdi-debug-step-into', 'mdi-debug-step-out', 'mdi-debug-step-over', 'mdi-decimal-decrease', 'mdi-decimal-increase', 'mdi-delete', 'mdi-delete-variant', 'mdi-delta', 'mdi-deskphone', 'mdi-desktop-mac', 'mdi-desktop-tower', 'mdi-details', 'mdi-deviantart', 'mdi-diamond', 'mdi-dice', 'mdi-dice-1', 'mdi-dice-2', 'mdi-dice-3', 'mdi-dice-4', 'mdi-dice-5', 'mdi-dice-6', 'mdi-directions', 'mdi-disk-alert', 'mdi-disqus', 'mdi-disqus-outline', 'mdi-division', 'mdi-division-box', 'mdi-dns', 'mdi-domain', 'mdi-dots-horizontal', 'mdi-dots-vertical', 'mdi-download', 'mdi-drag', 'mdi-drag-horizontal', 'mdi-drag-vertical', 'mdi-drawing', 'mdi-drawing-box', 'mdi-dribbble', 'mdi-dribbble-box', 'mdi-drone', 'mdi-dropbox', 'mdi-drupal', 'mdi-duck', 'mdi-dumbbell', 'mdi-earth', 'mdi-earth-off', 'mdi-edge', 'mdi-eject', 'mdi-elevation-decline', 'mdi-elevation-rise', 'mdi-elevator', 'mdi-email', 'mdi-email-open', 'mdi-email-outline', 'mdi-email-secure', 'mdi-emoticon', 'mdi-emoticon-cool', 'mdi-emoticon-devil', 'mdi-emoticon-happy', 'mdi-emoticon-neutral', 'mdi-emoticon-poop', 'mdi-emoticon-sad', 'mdi-emoticon-tongue', 'mdi-engine', 'mdi-engine-outline', 'mdi-equal', 'mdi-equal-box', 'mdi-eraser', 'mdi-escalator', 'mdi-ethernet', 'mdi-ethernet-cable', 'mdi-ethernet-cable-off', 'mdi-etsy', 'mdi-evernote', 'mdi-exclamation', 'mdi-exit-to-app', 'mdi-export', 'mdi-eye', 'mdi-eye-off', 'mdi-eyedropper', 'mdi-eyedropper-variant', 'mdi-facebook', 'mdi-facebook-box', 'mdi-facebook-messenger', 'mdi-factory', 'mdi-fan', 'mdi-fast-forward', 'mdi-fax', 'mdi-ferry', 'mdi-file', 'mdi-file-chart', 'mdi-file-check', 'mdi-file-cloud', 'mdi-file-delimited', 'mdi-file-document', 'mdi-file-document-box', 'mdi-file-excel', 'mdi-file-excel-box', 'mdi-file-export', 'mdi-file-find', 'mdi-file-image', 'mdi-file-import', 'mdi-file-lock', 'mdi-file-multiple', 'mdi-file-music', 'mdi-file-outline', 'mdi-file-pdf', 'mdi-file-pdf-box', 'mdi-file-powerpoint', 'mdi-file-powerpoint-box', 'mdi-file-presentation-box', 'mdi-file-send', 'mdi-file-video', 'mdi-file-word', 'mdi-file-word-box', 'mdi-file-xml', 'mdi-film', 'mdi-filmstrip', 'mdi-filmstrip-off', 'mdi-filter', 'mdi-filter-outline', 'mdi-filter-remove', 'mdi-filter-remove-outline', 'mdi-filter-variant', 'mdi-fingerprint', 'mdi-fire', 'mdi-firefox', 'mdi-fish', 'mdi-flag', 'mdi-flag-checkered', 'mdi-flag-outline', 'mdi-flag-outline-variant', 'mdi-flag-triangle', 'mdi-flag-variant', 'mdi-flash', 'mdi-flash-auto', 'mdi-flash-off', 'mdi-flashlight', 'mdi-flashlight-off', 'mdi-flattr', 'mdi-flip-to-back', 'mdi-flip-to-front', 'mdi-floppy', 'mdi-flower', 'mdi-folder', 'mdi-folder-account', 'mdi-folder-download', 'mdi-folder-google-drive', 'mdi-folder-image', 'mdi-folder-lock', 'mdi-folder-lock-open', 'mdi-folder-move', 'mdi-folder-multiple', 'mdi-folder-multiple-image', 'mdi-folder-multiple-outline', 'mdi-folder-outline', 'mdi-folder-plus', 'mdi-folder-remove', 'mdi-folder-upload', 'mdi-food', 'mdi-food-apple', 'mdi-food-variant', 'mdi-football', 'mdi-football-australian', 'mdi-football-helmet', 'mdi-format-align-center', 'mdi-format-align-justify', 'mdi-format-align-left', 'mdi-format-align-right', 'mdi-format-bold', 'mdi-format-clear', 'mdi-format-color-fill', 'mdi-format-float-center', 'mdi-format-float-left', 'mdi-format-float-none', 'mdi-format-float-right', 'mdi-format-header-1', 'mdi-format-header-2', 'mdi-format-header-3', 'mdi-format-header-4', 'mdi-format-header-5', 'mdi-format-header-6', 'mdi-format-header-decrease', 'mdi-format-header-equal', 'mdi-format-header-increase', 'mdi-format-header-pound', 'mdi-format-indent-decrease', 'mdi-format-indent-increase', 'mdi-format-italic', 'mdi-format-line-spacing', 'mdi-format-list-bulleted', 'mdi-format-list-bulleted-type', 'mdi-format-list-numbers', 'mdi-format-paint', 'mdi-format-paragraph', 'mdi-format-quote', 'mdi-format-size', 'mdi-format-strikethrough', 'mdi-format-strikethrough-variant', 'mdi-format-subscript', 'mdi-format-superscript', 'mdi-format-text', 'mdi-format-textdirection-l-to-r', 'mdi-format-textdirection-r-to-l', 'mdi-format-underline', 'mdi-format-wrap-inline', 'mdi-format-wrap-square', 'mdi-format-wrap-tight', 'mdi-format-wrap-top-bottom', 'mdi-forum', 'mdi-forward', 'mdi-foursquare', 'mdi-fridge', 'mdi-fridge-filled', 'mdi-fridge-filled-bottom', 'mdi-fridge-filled-top', 'mdi-fullscreen', 'mdi-fullscreen-exit', 'mdi-function', 'mdi-gamepad', 'mdi-gamepad-variant', 'mdi-gas-station', 'mdi-gate', 'mdi-gauge', 'mdi-gavel', 'mdi-gender-female', 'mdi-gender-male', 'mdi-gender-male-female', 'mdi-gender-transgender', 'mdi-ghost', 'mdi-gift', 'mdi-git', 'mdi-github-box', 'mdi-github-circle', 'mdi-glass-flute', 'mdi-glass-mug', 'mdi-glass-stange', 'mdi-glass-tulip', 'mdi-glasses', 'mdi-gmail', 'mdi-google', 'mdi-google-cardboard', 'mdi-google-chrome', 'mdi-google-circles', 'mdi-google-circles-communities', 'mdi-google-circles-extended', 'mdi-google-circles-group', 'mdi-google-controller', 'mdi-google-controller-off', 'mdi-google-drive', 'mdi-google-earth', 'mdi-google-glass', 'mdi-google-nearby', 'mdi-google-pages', 'mdi-google-physical-web', 'mdi-google-play', 'mdi-google-plus', 'mdi-google-plus-box', 'mdi-google-translate', 'mdi-google-wallet', 'mdi-grid', 'mdi-grid-off', 'mdi-group', 'mdi-guitar', 'mdi-guitar-pick', 'mdi-guitar-pick-outline', 'mdi-hand-pointing-right', 'mdi-hanger', 'mdi-hangouts', 'mdi-harddisk', 'mdi-headphones', 'mdi-headphones-box', 'mdi-headphones-settings', 'mdi-headset', 'mdi-headset-dock', 'mdi-headset-off', 'mdi-heart', 'mdi-heart-box', 'mdi-heart-box-outline', 'mdi-heart-broken', 'mdi-heart-outline', 'mdi-help', 'mdi-help-circle', 'mdi-hexagon', 'mdi-hexagon-outline', 'mdi-history', 'mdi-hololens', 'mdi-home', 'mdi-home-modern', 'mdi-home-variant', 'mdi-hops', 'mdi-hospital', 'mdi-hospital-building', 'mdi-hospital-marker', 'mdi-hotel', 'mdi-houzz', 'mdi-houzz-box', 'mdi-human', 'mdi-human-child', 'mdi-human-male-female', 'mdi-image', 'mdi-image-album', 'mdi-image-area', 'mdi-image-area-close', 'mdi-image-broken', 'mdi-image-broken-variant', 'mdi-image-filter', 'mdi-image-filter-black-white', 'mdi-image-filter-center-focus', 'mdi-image-filter-center-focus-weak', 'mdi-image-filter-drama', 'mdi-image-filter-frames', 'mdi-image-filter-hdr', 'mdi-image-filter-none', 'mdi-image-filter-tilt-shift', 'mdi-image-filter-vintage', 'mdi-image-multiple', 'mdi-import', 'mdi-inbox', 'mdi-information', 'mdi-information-outline', 'mdi-instagram', 'mdi-instapaper', 'mdi-internet-explorer', 'mdi-invert-colors', 'mdi-jeepney', 'mdi-jira', 'mdi-jsfiddle', 'mdi-keg', 'mdi-key', 'mdi-key-change', 'mdi-key-minus', 'mdi-key-plus', 'mdi-key-remove', 'mdi-key-variant', 'mdi-keyboard', 'mdi-keyboard-backspace', 'mdi-keyboard-caps', 'mdi-keyboard-close', 'mdi-keyboard-off', 'mdi-keyboard-return', 'mdi-keyboard-tab', 'mdi-keyboard-variant', 'mdi-label', 'mdi-label-outline', 'mdi-lan', 'mdi-lan-connect', 'mdi-lan-disconnect', 'mdi-lan-pending', 'mdi-language-csharp', 'mdi-language-css3', 'mdi-language-html5', 'mdi-language-javascript', 'mdi-language-php', 'mdi-language-python', 'mdi-language-python-text', 'mdi-laptop', 'mdi-laptop-chromebook', 'mdi-laptop-mac', 'mdi-laptop-windows', 'mdi-lastfm', 'mdi-launch', 'mdi-layers', 'mdi-layers-off', 'mdi-leaf', 'mdi-led-off', 'mdi-led-on', 'mdi-led-outline', 'mdi-led-variant-off', 'mdi-led-variant-on', 'mdi-led-variant-outline', 'mdi-library', 'mdi-library-books', 'mdi-library-music', 'mdi-library-plus', 'mdi-lightbulb', 'mdi-lightbulb-outline', 'mdi-link', 'mdi-link-off', 'mdi-link-variant', 'mdi-link-variant-off', 'mdi-linkedin', 'mdi-linkedin-box', 'mdi-linux', 'mdi-lock', 'mdi-lock-open', 'mdi-lock-open-outline', 'mdi-lock-outline', 'mdi-login', 'mdi-logout', 'mdi-looks', 'mdi-loupe', 'mdi-lumx', 'mdi-magnet', 'mdi-magnet-on', 'mdi-magnify', 'mdi-magnify-minus', 'mdi-magnify-plus', 'mdi-mail-ru', 'mdi-map', 'mdi-map-marker', 'mdi-map-marker-circle', 'mdi-map-marker-multiple', 'mdi-map-marker-off', 'mdi-map-marker-radius', 'mdi-margin', 'mdi-markdown', 'mdi-marker-check', 'mdi-martini', 'mdi-material-ui', 'mdi-math-compass', 'mdi-maxcdn', 'mdi-medium', 'mdi-memory', 'mdi-menu', 'mdi-menu-down', 'mdi-menu-left', 'mdi-menu-right', 'mdi-menu-up', 'mdi-message', 'mdi-message-alert', 'mdi-message-draw', 'mdi-message-image', 'mdi-message-outline', 'mdi-message-processing', 'mdi-message-reply', 'mdi-message-reply-text', 'mdi-message-text', 'mdi-message-text-outline', 'mdi-message-video', 'mdi-microphone', 'mdi-microphone-off', 'mdi-microphone-outline', 'mdi-microphone-settings', 'mdi-microphone-variant', 'mdi-microphone-variant-off', 'mdi-microsoft', 'mdi-minus', 'mdi-minus-box', 'mdi-minus-circle', 'mdi-minus-circle-outline', 'mdi-minus-network', 'mdi-monitor', 'mdi-monitor-multiple', 'mdi-more', 'mdi-motorbike', 'mdi-mouse', 'mdi-mouse-off', 'mdi-mouse-variant', 'mdi-mouse-variant-off', 'mdi-movie', 'mdi-multiplication', 'mdi-multiplication-box', 'mdi-music-box', 'mdi-music-box-outline', 'mdi-music-circle', 'mdi-music-note', 'mdi-music-note-eighth', 'mdi-music-note-half', 'mdi-music-note-off', 'mdi-music-note-quarter', 'mdi-music-note-sixteenth', 'mdi-music-note-whole', 'mdi-nature', 'mdi-nature-people', 'mdi-navigation', 'mdi-needle', 'mdi-nest-protect', 'mdi-nest-thermostat', 'mdi-newspaper', 'mdi-nfc', 'mdi-nfc-tap', 'mdi-nfc-variant', 'mdi-nodejs', 'mdi-note', 'mdi-note-outline', 'mdi-note-plus', 'mdi-note-plus-outline', 'mdi-note-text', 'mdi-notification-clear-all', 'mdi-numeric', 'mdi-numeric-0-box', 'mdi-numeric-0-box-multiple-outline', 'mdi-numeric-0-box-outline', 'mdi-numeric-1-box', 'mdi-numeric-1-box-multiple-outline', 'mdi-numeric-1-box-outline', 'mdi-numeric-2-box', 'mdi-numeric-2-box-multiple-outline', 'mdi-numeric-2-box-outline', 'mdi-numeric-3-box', 'mdi-numeric-3-box-multiple-outline', 'mdi-numeric-3-box-outline', 'mdi-numeric-4-box', 'mdi-numeric-4-box-multiple-outline', 'mdi-numeric-4-box-outline', 'mdi-numeric-5-box', 'mdi-numeric-5-box-multiple-outline', 'mdi-numeric-5-box-outline', 'mdi-numeric-6-box', 'mdi-numeric-6-box-multiple-outline', 'mdi-numeric-6-box-outline', 'mdi-numeric-7-box', 'mdi-numeric-7-box-multiple-outline', 'mdi-numeric-7-box-outline', 'mdi-numeric-8-box', 'mdi-numeric-8-box-multiple-outline', 'mdi-numeric-8-box-outline', 'mdi-numeric-9-box', 'mdi-numeric-9-box-multiple-outline', 'mdi-numeric-9-box-outline', 'mdi-numeric-9-plus-box', 'mdi-numeric-9-plus-box-multiple-outline', 'mdi-numeric-9-plus-box-outline', 'mdi-nutrition', 'mdi-octagon', 'mdi-octagon-outline', 'mdi-odnoklassniki', 'mdi-office', 'mdi-oil', 'mdi-oil-temperature', 'mdi-omega', 'mdi-onedrive', 'mdi-open-in-app', 'mdi-open-in-new', 'mdi-opera', 'mdi-ornament', 'mdi-ornament-variant', 'mdi-outbox', 'mdi-owl', 'mdi-package', 'mdi-package-down', 'mdi-package-up', 'mdi-package-variant', 'mdi-package-variant-closed', 'mdi-palette', 'mdi-palette-advanced', 'mdi-panda', 'mdi-pandora', 'mdi-panorama', 'mdi-panorama-fisheye', 'mdi-panorama-horizontal', 'mdi-panorama-vertical', 'mdi-panorama-wide-angle', 'mdi-paper-cut-vertical', 'mdi-paperclip', 'mdi-parking', 'mdi-pause', 'mdi-pause-circle', 'mdi-pause-circle-outline', 'mdi-pause-octagon', 'mdi-pause-octagon-outline', 'mdi-paw', 'mdi-pen', 'mdi-pencil', 'mdi-pencil-box', 'mdi-pencil-box-outline', 'mdi-pencil-lock', 'mdi-pencil-off', 'mdi-percent', 'mdi-pharmacy', 'mdi-phone', 'mdi-phone-bluetooth', 'mdi-phone-forward', 'mdi-phone-hangup', 'mdi-phone-in-talk', 'mdi-phone-incoming', 'mdi-phone-locked', 'mdi-phone-log', 'mdi-phone-missed', 'mdi-phone-outgoing', 'mdi-phone-paused', 'mdi-phone-settings', 'mdi-phone-voip', 'mdi-pi', 'mdi-pi-box', 'mdi-pig', 'mdi-pill', 'mdi-pin', 'mdi-pin-off', 'mdi-pine-tree', 'mdi-pine-tree-box', 'mdi-pinterest', 'mdi-pinterest-box', 'mdi-pizza', 'mdi-play', 'mdi-play-box-outline', 'mdi-play-circle', 'mdi-play-circle-outline', 'mdi-play-pause', 'mdi-play-protected-content', 'mdi-playlist-minus', 'mdi-playlist-play', 'mdi-playlist-plus', 'mdi-playlist-remove', 'mdi-playstation', 'mdi-plus', 'mdi-plus-box', 'mdi-plus-circle', 'mdi-plus-circle-multiple-outline', 'mdi-plus-circle-outline', 'mdi-plus-network', 'mdi-plus-one', 'mdi-pocket', 'mdi-pokeball', 'mdi-polaroid', 'mdi-poll', 'mdi-poll-box', 'mdi-polymer', 'mdi-popcorn', 'mdi-pound', 'mdi-pound-box', 'mdi-power', 'mdi-power-settings', 'mdi-power-socket', 'mdi-presentation', 'mdi-presentation-play', 'mdi-printer', 'mdi-printer-3d', 'mdi-printer-alert', 'mdi-professional-hexagon', 'mdi-projector', 'mdi-projector-screen', 'mdi-pulse', 'mdi-puzzle', 'mdi-qrcode', 'mdi-qrcode-scan', 'mdi-quadcopter', 'mdi-quality-high', 'mdi-quicktime', 'mdi-radar', 'mdi-radiator', 'mdi-radio', 'mdi-radio-handheld', 'mdi-radio-tower', 'mdi-radioactive', 'mdi-radiobox-blank', 'mdi-radiobox-marked', 'mdi-raspberrypi', 'mdi-ray-end', 'mdi-ray-end-arrow', 'mdi-ray-start', 'mdi-ray-start-arrow', 'mdi-ray-start-end', 'mdi-ray-vertex', 'mdi-rdio', 'mdi-read', 'mdi-readability', 'mdi-receipt', 'mdi-record', 'mdi-record-rec', 'mdi-recycle', 'mdi-reddit', 'mdi-redo', 'mdi-redo-variant', 'mdi-refresh', 'mdi-regex', 'mdi-relative-scale', 'mdi-reload', 'mdi-remote', 'mdi-rename-box', 'mdi-repeat', 'mdi-repeat-off', 'mdi-repeat-once', 'mdi-replay', 'mdi-reply', 'mdi-reply-all', 'mdi-reproduction', 'mdi-resize-bottom-right', 'mdi-responsive', 'mdi-rewind', 'mdi-ribbon', 'mdi-road', 'mdi-road-variant', 'mdi-rocket', 'mdi-rotate-3d', 'mdi-rotate-left', 'mdi-rotate-left-variant', 'mdi-rotate-right', 'mdi-rotate-right-variant', 'mdi-router-wireless', 'mdi-routes', 'mdi-rss', 'mdi-rss-box', 'mdi-ruler', 'mdi-run', 'mdi-sale', 'mdi-satellite', 'mdi-satellite-variant', 'mdi-scale', 'mdi-scale-bathroom', 'mdi-school', 'mdi-screen-rotation', 'mdi-screen-rotation-lock', 'mdi-screwdriver', 'mdi-script', 'mdi-sd', 'mdi-seal', 'mdi-seat-flat', 'mdi-seat-flat-angled', 'mdi-seat-individual-suite', 'mdi-seat-legroom-extra', 'mdi-seat-legroom-normal', 'mdi-seat-legroom-reduced', 'mdi-seat-recline-extra', 'mdi-seat-recline-normal', 'mdi-security', 'mdi-security-network', 'mdi-select', 'mdi-select-all', 'mdi-select-inverse', 'mdi-select-off', 'mdi-selection', 'mdi-send', 'mdi-server', 'mdi-server-minus', 'mdi-server-network', 'mdi-server-network-off', 'mdi-server-off', 'mdi-server-plus', 'mdi-server-remove', 'mdi-server-security', 'mdi-settings', 'mdi-settings-box', 'mdi-shape-plus', 'mdi-share', 'mdi-share-variant', 'mdi-shield', 'mdi-shield-outline', 'mdi-shopping', 'mdi-shopping-music', 'mdi-shredder', 'mdi-shuffle', 'mdi-shuffle-disabled', 'mdi-shuffle-variant', 'mdi-sigma', 'mdi-sign-caution', 'mdi-signal', 'mdi-silverware', 'mdi-silverware-fork', 'mdi-silverware-spoon', 'mdi-silverware-variant', 'mdi-sim', 'mdi-sim-alert', 'mdi-sim-off', 'mdi-sitemap', 'mdi-skip-backward', 'mdi-skip-forward', 'mdi-skip-next', 'mdi-skip-previous', 'mdi-skype', 'mdi-skype-business', 'mdi-slack', 'mdi-sleep', 'mdi-sleep-off', 'mdi-smoking', 'mdi-smoking-off', 'mdi-snapchat', 'mdi-snowman', 'mdi-sofa', 'mdi-sort', 'mdi-sort-alphabetical', 'mdi-sort-ascending', 'mdi-sort-descending', 'mdi-sort-numeric', 'mdi-sort-variant', 'mdi-soundcloud', 'mdi-source-fork', 'mdi-source-pull', 'mdi-speaker', 'mdi-speaker-off', 'mdi-speedometer', 'mdi-spellcheck', 'mdi-spotify', 'mdi-spotlight', 'mdi-spotlight-beam', 'mdi-square-inc', 'mdi-square-inc-cash', 'mdi-stackoverflow', 'mdi-stairs', 'mdi-star', 'mdi-star-circle', 'mdi-star-half', 'mdi-star-off', 'mdi-star-outline', 'mdi-steam', 'mdi-steering', 'mdi-step-backward', 'mdi-step-backward-2', 'mdi-step-forward', 'mdi-step-forward-2', 'mdi-stethoscope', 'mdi-stocking', 'mdi-stop', 'mdi-store', 'mdi-store-24-hour', 'mdi-stove', 'mdi-subway', 'mdi-sunglasses', 'mdi-swap-horizontal', 'mdi-swap-vertical', 'mdi-swim', 'mdi-switch', 'mdi-sword', 'mdi-sync', 'mdi-sync-alert', 'mdi-sync-off', 'mdi-tab', 'mdi-tab-unselected', 'mdi-table', 'mdi-table-column-plus-after', 'mdi-table-column-plus-before', 'mdi-table-column-remove', 'mdi-table-column-width', 'mdi-table-edit', 'mdi-table-large', 'mdi-table-row-height', 'mdi-table-row-plus-after', 'mdi-table-row-plus-before', 'mdi-table-row-remove', 'mdi-tablet', 'mdi-tablet-android', 'mdi-tablet-ipad', 'mdi-tag', 'mdi-tag-faces', 'mdi-tag-multiple', 'mdi-tag-outline', 'mdi-tag-text-outline', 'mdi-target', 'mdi-taxi', 'mdi-teamviewer', 'mdi-telegram', 'mdi-television', 'mdi-television-guide', 'mdi-temperature-celsius', 'mdi-temperature-fahrenheit', 'mdi-temperature-kelvin', 'mdi-tennis', 'mdi-tent', 'mdi-terrain', 'mdi-text-to-speech', 'mdi-text-to-speech-off', 'mdi-texture', 'mdi-theater', 'mdi-theme-light-dark', 'mdi-thermometer', 'mdi-thermometer-lines', 'mdi-thumb-down', 'mdi-thumb-down-outline', 'mdi-thumb-up', 'mdi-thumb-up-outline', 'mdi-thumbs-up-down', 'mdi-ticket', 'mdi-ticket-account', 'mdi-ticket-confirmation', 'mdi-tie', 'mdi-timelapse', 'mdi-timer', 'mdi-timer-10', 'mdi-timer-3', 'mdi-timer-off', 'mdi-timer-sand', 'mdi-timetable', 'mdi-toggle-switch', 'mdi-toggle-switch-off', 'mdi-tooltip', 'mdi-tooltip-edit', 'mdi-tooltip-image', 'mdi-tooltip-outline', 'mdi-tooltip-outline-plus', 'mdi-tooltip-text', 'mdi-tor', 'mdi-traffic-light', 'mdi-train', 'mdi-tram', 'mdi-transcribe', 'mdi-transcribe-close', 'mdi-transfer', 'mdi-tree', 'mdi-trello', 'mdi-trending-down', 'mdi-trending-neutral', 'mdi-trending-up', 'mdi-triangle', 'mdi-triangle-outline', 'mdi-trophy', 'mdi-trophy-award', 'mdi-trophy-outline', 'mdi-trophy-variant', 'mdi-trophy-variant-outline', 'mdi-truck', 'mdi-truck-delivery', 'mdi-tshirt-crew', 'mdi-tshirt-v', 'mdi-tumblr', 'mdi-tumblr-reblog', 'mdi-twitch', 'mdi-twitter', 'mdi-twitter-box', 'mdi-twitter-circle', 'mdi-twitter-retweet', 'mdi-ubuntu', 'mdi-umbraco', 'mdi-umbrella', 'mdi-umbrella-outline', 'mdi-undo', 'mdi-undo-variant', 'mdi-unfold-less', 'mdi-unfold-more', 'mdi-ungroup', 'mdi-untappd', 'mdi-upload', 'mdi-usb', 'mdi-vector-arrange-above', 'mdi-vector-arrange-below', 'mdi-vector-circle', 'mdi-vector-circle-variant', 'mdi-vector-combine', 'mdi-vector-curve', 'mdi-vector-difference', 'mdi-vector-difference-ab', 'mdi-vector-difference-ba', 'mdi-vector-intersection', 'mdi-vector-line', 'mdi-vector-point', 'mdi-vector-polygon', 'mdi-vector-polyline', 'mdi-vector-selection', 'mdi-vector-square', 'mdi-vector-triangle', 'mdi-vector-union', 'mdi-verified', 'mdi-vibrate', 'mdi-video', 'mdi-video-off', 'mdi-video-switch', 'mdi-view-agenda', 'mdi-view-array', 'mdi-view-carousel', 'mdi-view-column', 'mdi-view-dashboard', 'mdi-view-day', 'mdi-view-grid', 'mdi-view-headline', 'mdi-view-list', 'mdi-view-module', 'mdi-view-quilt', 'mdi-view-stream', 'mdi-view-week', 'mdi-vimeo', 'mdi-vine', 'mdi-vk', 'mdi-vk-box', 'mdi-vk-circle', 'mdi-voicemail', 'mdi-volume-high', 'mdi-volume-low', 'mdi-volume-medium', 'mdi-volume-off', 'mdi-vpn', 'mdi-walk', 'mdi-wallet', 'mdi-wallet-giftcard', 'mdi-wallet-membership', 'mdi-wallet-travel', 'mdi-wan', 'mdi-watch', 'mdi-watch-export', 'mdi-watch-import', 'mdi-water', 'mdi-water-off', 'mdi-water-percent', 'mdi-water-pump', 'mdi-weather-cloudy', 'mdi-weather-fog', 'mdi-weather-hail', 'mdi-weather-lightning', 'mdi-weather-night', 'mdi-weather-partlycloudy', 'mdi-weather-pouring', 'mdi-weather-rainy', 'mdi-weather-snowy', 'mdi-weather-sunny', 'mdi-weather-sunset', 'mdi-weather-sunset-down', 'mdi-weather-sunset-up', 'mdi-weather-windy', 'mdi-weather-windy-variant', 'mdi-web', 'mdi-webcam', 'mdi-weight', 'mdi-weight-kilogram', 'mdi-whatsapp', 'mdi-wheelchair-accessibility', 'mdi-white-balance-auto', 'mdi-white-balance-incandescent', 'mdi-white-balance-irradescent', 'mdi-white-balance-sunny', 'mdi-wifi', 'mdi-wifi-off', 'mdi-wii', 'mdi-wikipedia', 'mdi-window-close', 'mdi-window-closed', 'mdi-window-maximize', 'mdi-window-minimize', 'mdi-window-open', 'mdi-window-restore', 'mdi-windows', 'mdi-wordpress', 'mdi-worker', 'mdi-wrap', 'mdi-wrench', 'mdi-wunderlist', 'mdi-xbox', 'mdi-xbox-controller', 'mdi-xbox-controller-off', 'mdi-xda', 'mdi-xing', 'mdi-xing-box', 'mdi-xing-circle', 'mdi-xml', 'mdi-yeast', 'mdi-yelp', 'mdi-youtube-play', 'mdi-zip-box')
    PACK_MATERIAL = ('md-3d-rotation', 'md-access-alarm', 'md-access-alarms', 'md-access-time', 'md-accessibility', 'md-account-balance', 'md-account-balance-wallet', 'md-account-box', 'md-account-circle', 'md-adb', 'md-add', 'md-add-alarm', 'md-add-alert', 'md-add-box', 'md-add-circle', 'md-add-circle-outline', 'md-add-shopping-cart', 'md-add-to-photos', 'md-adjust', 'md-airline-seat-flat', 'md-airline-seat-flat-angled', 'md-airline-seat-individual-suite', 'md-airline-seat-legroom-extra', 'md-airline-seat-legroom-normal', 'md-airline-seat-legroom-reduced', 'md-airline-seat-recline-extra', 'md-airline-seat-recline-normal', 'md-airplanemode-active', 'md-airplanemode-inactive', 'md-airplay', 'md-alarm', 'md-alarm-add', 'md-alarm-off', 'md-alarm-on', 'md-album', 'md-android', 'md-announcement', 'md-apps', 'md-archive', 'md-arrow-back', 'md-arrow-drop-down', 'md-arrow-drop-down-circle', 'md-arrow-drop-up', 'md-arrow-forward', 'md-aspect-ratio', 'md-assessment', 'md-assignment', 'md-assignment-ind', 'md-assignment-late', 'md-assignment-return', 'md-assignment-returned', 'md-assignment-turned-in', 'md-assistant', 'md-assistant-photo', 'md-attach-file', 'md-attach-money', 'md-attachment', 'md-audiotrack', 'md-autorenew', 'md-av-timer', 'md-backspace', 'md-backup', 'md-battery-alert', 'md-battery-charging-full', 'md-battery-full', 'md-battery-std', 'md-battery-unknown', 'md-beenhere', 'md-block', 'md-bluetooth', 'md-bluetooth-audio', 'md-bluetooth-connected', 'md-bluetooth-disabled', 'md-bluetooth-searching', 'md-blur-circular', 'md-blur-linear', 'md-blur-off', 'md-blur-on', 'md-book', 'md-bookmark', 'md-bookmark-border', 'md-border-all', 'md-border-bottom', 'md-border-clear', 'md-border-color', 'md-border-horizontal', 'md-border-inner', 'md-border-left', 'md-border-outer', 'md-border-right', 'md-border-style', 'md-border-top', 'md-border-vertical', 'md-brightness-1', 'md-brightness-2', 'md-brightness-3', 'md-brightness-4', 'md-brightness-5', 'md-brightness-6', 'md-brightness-7', 'md-brightness-auto', 'md-brightness-high', 'md-brightness-low', 'md-brightness-medium', 'md-broken-image', 'md-brush', 'md-bug-report', 'md-build', 'md-business', 'md-cached', 'md-cake', 'md-call', 'md-call-end', 'md-call-made', 'md-call-merge', 'md-call-missed', 'md-call-received', 'md-call-split', 'md-camera', 'md-camera-alt', 'md-camera-enhance', 'md-camera-front', 'md-camera-rear', 'md-camera-roll', 'md-cancel', 'md-card-giftcard', 'md-card-membership', 'md-card-travel', 'md-cast', 'md-cast-connected', 'md-center-focus-strong', 'md-center-focus-weak', 'md-change-history', 'md-chat', 'md-chat-bubble', 'md-chat-bubble-outline', 'md-check', 'md-check-box', 'md-check-box-outline-blank', 'md-check-circle', 'md-chevron-left', 'md-chevron-right', 'md-chrome-reader-mode', 'md-class', 'md-clear', 'md-clear-all', 'md-close', 'md-closed-caption', 'md-cloud', 'md-cloud-circle', 'md-cloud-done', 'md-cloud-download', 'md-cloud-off', 'md-cloud-queue', 'md-cloud-upload', 'md-code', 'md-collections', 'md-collections-bookmark', 'md-color-lens', 'md-colorize', 'md-comment', 'md-compare', 'md-computer', 'md-confirmation-number', 'md-contact-phone', 'md-contacts', 'md-content-copy', 'md-content-cut', 'md-content-paste', 'md-control-point', 'md-control-point-duplicate', 'md-create', 'md-credit-card', 'md-crop', 'md-crop-16-9', 'md-crop-3-2', 'md-crop-5-4', 'md-crop-7-5', 'md-crop-din', 'md-crop-free', 'md-crop-landscape', 'md-crop-original', 'md-crop-portrait', 'md-crop-square', 'md-dashboard', 'md-data-usage', 'md-dehaze', 'md-delete', 'md-description', 'md-desktop-mac', 'md-desktop-windows', 'md-details', 'md-developer-board', 'md-developer-mode', 'md-device-hub', 'md-devices', 'md-dialer-sip', 'md-dialpad', 'md-directions', 'md-directions-bike', 'md-directions-boat', 'md-directions-bus', 'md-directions-car', 'md-directions-railway', 'md-directions-run', 'md-directions-subway', 'md-directions-transit', 'md-directions-walk', 'md-disc-full', 'md-dns', 'md-do-not-disturb', 'md-do-not-disturb-alt', 'md-dock', 'md-domain', 'md-done', 'md-done-all', 'md-drafts', 'md-drive-eta', 'md-dvr', 'md-edit', 'md-eject', 'md-email', 'md-equalizer', 'md-error', 'md-error-outline', 'md-event', 'md-event-available', 'md-event-busy', 'md-event-note', 'md-event-seat', 'md-exit-to-app', 'md-expand-less', 'md-expand-more', 'md-explicit', 'md-explore', 'md-exposure', 'md-exposure-neg-1', 'md-exposure-neg-2', 'md-exposure-plus-1', 'md-exposure-plus-2', 'md-exposure-zero', 'md-extension', 'md-face', 'md-fast-forward', 'md-fast-rewind', 'md-favorite', 'md-favorite-border', 'md-feedback', 'md-file-download', 'md-file-upload', 'md-filter', 'md-filter-1', 'md-filter-2', 'md-filter-3', 'md-filter-4', 'md-filter-5', 'md-filter-6', 'md-filter-7', 'md-filter-8', 'md-filter-9', 'md-filter-9-plus', 'md-filter-b-and-w', 'md-filter-center-focus', 'md-filter-drama', 'md-filter-frames', 'md-filter-hdr', 'md-filter-list', 'md-filter-none', 'md-filter-tilt-shift', 'md-filter-vintage', 'md-find-in-page', 'md-find-replace', 'md-flag', 'md-flare', 'md-flash-auto', 'md-flash-off', 'md-flash-on', 'md-flight', 'md-flight-land', 'md-flight-takeoff', 'md-flip', 'md-flip-to-back', 'md-flip-to-front', 'md-folder', 'md-folder-open', 'md-folder-shared', 'md-folder-special', 'md-font-download', 'md-format-align-center', 'md-format-align-justify', 'md-format-align-left', 'md-format-align-right', 'md-format-bold', 'md-format-clear', 'md-format-color-fill', 'md-format-color-reset', 'md-format-color-text', 'md-format-indent-decrease', 'md-format-indent-increase', 'md-format-italic', 'md-format-line-spacing', 'md-format-list-bulleted', 'md-format-list-numbered', 'md-format-paint', 'md-format-quote', 'md-format-size', 'md-format-strikethrough', 'md-format-textdirection-l-to-r', 'md-format-textdirection-r-to-l', 'md-format-underlined', 'md-forum', 'md-forward', 'md-forward-10', 'md-forward-30', 'md-forward-5', 'md-fullscreen', 'md-fullscreen-exit', 'md-functions', 'md-gamepad', 'md-games', 'md-gesture', 'md-get-app', 'md-gif', 'md-gps-fixed', 'md-gps-not-fixed', 'md-gps-off', 'md-grade', 'md-gradient', 'md-grain', 'md-graphic-eq', 'md-grid-off', 'md-grid-on', 'md-group', 'md-group-add', 'md-group-work', 'md-hd', 'md-hdr-off', 'md-hdr-on', 'md-hdr-strong', 'md-hdr-weak', 'md-headset', 'md-headset-mic', 'md-healing', 'md-hearing', 'md-help', 'md-help-outline', 'md-high-quality', 'md-highlight-off', 'md-history', 'md-home', 'md-hotel', 'md-hourglass-empty', 'md-hourglass-full', 'md-http', 'md-https', 'md-image', 'md-image-aspect-ratio', 'md-import-export', 'md-inbox', 'md-indeterminate-check-box', 'md-info', 'md-info-outline', 'md-input', 'md-insert-chart', 'md-insert-comment', 'md-insert-drive-file', 'md-insert-emoticon', 'md-insert-invitation', 'md-insert-link', 'md-insert-photo', 'md-invert-colors', 'md-invert-colors-off', 'md-iso', 'md-keyboard', 'md-keyboard-arrow-down', 'md-keyboard-arrow-left', 'md-keyboard-arrow-right', 'md-keyboard-arrow-up', 'md-keyboard-backspace', 'md-keyboard-capslock', 'md-keyboard-hide', 'md-keyboard-return', 'md-keyboard-tab', 'md-keyboard-voice', 'md-label', 'md-label-outline', 'md-landscape', 'md-language', 'md-laptop', 'md-laptop-chromebook', 'md-laptop-mac', 'md-laptop-windows', 'md-launch', 'md-layers', 'md-layers-clear', 'md-leak-add', 'md-leak-remove', 'md-lens', 'md-library-add', 'md-library-books', 'md-library-music', 'md-link', 'md-list', 'md-live-help', 'md-live-tv', 'md-local-activity', 'md-local-airport', 'md-local-atm', 'md-local-bar', 'md-local-cafe', 'md-local-car-wash', 'md-local-convenience-store', 'md-local-dining', 'md-local-drink', 'md-local-florist', 'md-local-gas-station', 'md-local-grocery-store', 'md-local-hospital', 'md-local-hotel', 'md-local-laundry-service', 'md-local-library', 'md-local-mall', 'md-local-movies', 'md-local-offer', 'md-local-parking', 'md-local-pharmacy', 'md-local-phone', 'md-local-pizza', 'md-local-play', 'md-local-post-office', 'md-local-printshop', 'md-local-see', 'md-local-shipping', 'md-local-taxi', 'md-location-city', 'md-location-disabled', 'md-location-off', 'md-location-on', 'md-location-searching', 'md-lock', 'md-lock-open', 'md-lock-outline', 'md-looks', 'md-looks-3', 'md-looks-4', 'md-looks-5', 'md-looks-6', 'md-looks-one', 'md-looks-two', 'md-loop', 'md-loupe', 'md-loyalty', 'md-mail', 'md-map', 'md-markunread', 'md-markunread-mailbox', 'md-memory', 'md-menu', 'md-merge-type', 'md-message', 'md-mic', 'md-mic-none', 'md-mic-off', 'md-mms', 'md-mode-comment', 'md-mode-edit', 'md-money-off', 'md-monochrome-photos', 'md-mood', 'md-mood-bad', 'md-more', 'md-more-horiz', 'md-more-vert', 'md-mouse', 'md-movie', 'md-movie-creation', 'md-music-note', 'md-my-location', 'md-nature', 'md-nature-people', 'md-navigate-before', 'md-navigate-next', 'md-navigation', 'md-network-cell', 'md-network-locked', 'md-network-wifi', 'md-new-releases', 'md-nfc', 'md-no-sim', 'md-not-interested', 'md-note-add', 'md-notifications', 'md-notifications-active', 'md-notifications-none', 'md-notifications-off', 'md-notifications-paused', 'md-offline-pin', 'md-ondemand-video', 'md-open-in-browser', 'md-open-in-new', 'md-open-with', 'md-pages', 'md-pageview', 'md-palette', 'md-panorama', 'md-panorama-fish-eye', 'md-panorama-horizontal', 'md-panorama-vertical', 'md-panorama-wide-angle', 'md-party-mode', 'md-pause', 'md-pause-circle-filled', 'md-pause-circle-outline', 'md-payment', 'md-people', 'md-people-outline', 'md-perm-camera-mic', 'md-perm-contact-calendar', 'md-perm-data-setting', 'md-perm-device-information', 'md-perm-identity', 'md-perm-media', 'md-perm-phone-msg', 'md-perm-scan-wifi', 'md-person', 'md-person-add', 'md-person-outline', 'md-person-pin', 'md-personal-video', 'md-phone', 'md-phone-android', 'md-phone-bluetooth-speaker', 'md-phone-forwarded', 'md-phone-in-talk', 'md-phone-iphone', 'md-phone-locked', 'md-phone-missed', 'md-phone-paused', 'md-phonelink', 'md-phonelink-erase', 'md-phonelink-lock', 'md-phonelink-off', 'md-phonelink-ring', 'md-phonelink-setup', 'md-photo', 'md-photo-album', 'md-photo-camera', 'md-photo-library', 'md-photo-size-select-actual', 'md-photo-size-select-large', 'md-photo-size-select-small', 'md-picture-as-pdf', 'md-picture-in-picture', 'md-pin-drop', 'md-place', 'md-play-arrow', 'md-play-circle-filled', 'md-play-circle-outline', 'md-play-for-work', 'md-playlist-add', 'md-plus-one', 'md-poll', 'md-polymer', 'md-portable-wifi-off', 'md-portrait', 'md-power', 'md-power-input', 'md-power-settings-new', 'md-present-to-all', 'md-print', 'md-public', 'md-publish', 'md-query-builder', 'md-question-answer', 'md-queue', 'md-queue-music', 'md-radio', 'md-radio-button-checked', 'md-radio-button-unchecked', 'md-rate-review', 'md-receipt', 'md-recent-actors', 'md-redeem', 'md-redo', 'md-refresh', 'md-remove', 'md-remove-circle', 'md-remove-circle-outline', 'md-remove-red-eye', 'md-reorder', 'md-repeat', 'md-repeat-one', 'md-replay', 'md-replay-10', 'md-replay-30', 'md-replay-5', 'md-reply', 'md-reply-all', 'md-report', 'md-report-problem', 'md-restaurant-menu', 'md-restore', 'md-ring-volume', 'md-room', 'md-rotate-90-degrees-ccw', 'md-rotate-left', 'md-rotate-right', 'md-router', 'md-satellite', 'md-save', 'md-scanner', 'md-schedule', 'md-school', 'md-screen-lock-landscape', 'md-screen-lock-portrait', 'md-screen-lock-rotation', 'md-screen-rotation', 'md-sd-card', 'md-sd-storage', 'md-search', 'md-security', 'md-select-all', 'md-send', 'md-settings', 'md-settings-applications', 'md-settings-backup-restore', 'md-settings-bluetooth', 'md-settings-brightness', 'md-settings-cell', 'md-settings-ethernet', 'md-settings-input-antenna', 'md-settings-input-component', 'md-settings-input-composite', 'md-settings-input-hdmi', 'md-settings-input-svideo', 'md-settings-overscan', 'md-settings-phone', 'md-settings-power', 'md-settings-remote', 'md-settings-system-daydream', 'md-settings-voice', 'md-share', 'md-shop', 'md-shop-two', 'md-shopping-basket', 'md-shopping-cart', 'md-shuffle', 'md-signal-cellular-4-bar', 'md-signal-cellular-connected-no-internet-4-bar', 'md-signal-cellular-no-sim', 'md-signal-cellular-null', 'md-signal-cellular-off', 'md-signal-wifi-4-bar', 'md-signal-wifi-4-bar-lock', 'md-signal-wifi-off', 'md-sim-card', 'md-sim-card-alert', 'md-skip-next', 'md-skip-previous', 'md-slideshow', 'md-smartphone', 'md-sms', 'md-sms-failed', 'md-snooze', 'md-sort', 'md-sort-by-alpha', 'md-space-bar', 'md-speaker', 'md-speaker-group', 'md-speaker-notes', 'md-speaker-phone', 'md-spellcheck', 'md-star', 'md-star-border', 'md-star-half', 'md-stars', 'md-stay-current-landscape', 'md-stay-current-portrait', 'md-stay-primary-landscape', 'md-stay-primary-portrait', 'md-stop', 'md-storage', 'md-store', 'md-store-mall-directory', 'md-straighten', 'md-strikethrough-s', 'md-style', 'md-subject', 'md-subtitles', 'md-supervisor-account', 'md-surround-sound', 'md-swap-calls', 'md-swap-horiz', 'md-swap-vert', 'md-swap-vertical-circle', 'md-switch-camera', 'md-switch-video', 'md-sync', 'md-sync-disabled', 'md-sync-problem', 'md-system-update', 'md-system-update-alt', 'md-tab', 'md-tab-unselected', 'md-tablet', 'md-tablet-android', 'md-tablet-mac', 'md-tag-faces', 'md-tap-and-play', 'md-terrain', 'md-text-format', 'md-textsms', 'md-texture', 'md-theaters', 'md-thumb-down', 'md-thumb-up', 'md-thumbs-up-down', 'md-time-to-leave', 'md-timelapse', 'md-timer', 'md-timer-10', 'md-timer-3', 'md-timer-off', 'md-toc', 'md-today', 'md-toll', 'md-tonality', 'md-toys', 'md-track-changes', 'md-traffic', 'md-transform', 'md-translate', 'md-trending-down', 'md-trending-flat', 'md-trending-up', 'md-tune', 'md-turned-in', 'md-turned-in-not', 'md-tv', 'md-undo', 'md-unfold-less', 'md-unfold-more', 'md-usb', 'md-verified-user', 'md-vertical-align-bottom', 'md-vertical-align-center', 'md-vertical-align-top', 'md-vibration', 'md-video-library', 'md-videocam', 'md-videocam-off', 'md-view-agenda', 'md-view-array', 'md-view-carousel', 'md-view-column', 'md-view-comfy', 'md-view-compact', 'md-view-day', 'md-view-headline', 'md-view-list', 'md-view-module', 'md-view-quilt', 'md-view-stream', 'md-view-week', 'md-vignette', 'md-visibility', 'md-visibility-off', 'md-voice-chat', 'md-voicemail', 'md-volume-down', 'md-volume-mute', 'md-volume-off', 'md-volume-up', 'md-vpn-key', 'md-vpn-lock', 'md-wallpaper', 'md-warning', 'md-watch', 'md-wb-auto', 'md-wb-cloudy', 'md-wb-incandescent', 'md-wb-iridescent', 'md-wb-sunny', 'md-wc', 'md-web', 'md-whatshot', 'md-widgets', 'md-wifi', 'md-wifi-lock', 'md-wifi-tethering', 'md-work', 'md-wrap-text', 'md-youtube-searched-for', 'md-zoom-in', 'md-zoom-out')
    PACK_METEOCONS = ('mc-celsius', 'mc-cloud', 'mc-cloud-double', 'mc-cloud-double-o', 'mc-cloud-double-thunder', 'mc-cloud-double-thunder-o', 'mc-cloud-double-thunder2-o', 'mc-cloud-drop', 'mc-cloud-drop-o', 'mc-cloud-o', 'mc-cloud-rain', 'mc-cloud-rain-o', 'mc-cloud-rain2', 'mc-cloud-rain2-o', 'mc-cloud-sea-o', 'mc-cloud-snow', 'mc-cloud-snow-o', 'mc-cloud-snow2', 'mc-cloud-snow2-o', 'mc-cloud-snow3-o', 'mc-cloud-thunder', 'mc-cloud-thunder-o', 'mc-cloud-thunder2-o', 'mc-cloud-wind', 'mc-cloud-wind-o', 'mc-cloud-wind-rain', 'mc-cloud-wind-rain-o', 'mc-cloudy-o', 'mc-compass', 'mc-eclipse', 'mc-fahrenheit', 'mc-moon', 'mc-moon-cloud', 'mc-moon-cloud-o', 'mc-moon-o', 'mc-moonrise-sea-o', 'mc-not-applicable', 'mc-sea-o', 'mc-snow', 'mc-sun', 'mc-sun-cloud', 'mc-sun-cloud-o', 'mc-sun-o', 'mc-sunrise-o', 'mc-sunrise-sea-o', 'mc-thermometer', 'mc-wind-o')
    PACK_WEATHER = ('wi_alien', 'wi_barometer', 'wi_celsius', 'wi_cloud', 'wi_cloud_down', 'wi_cloud_refresh', 'wi_cloud_up', 'wi_cloudy', 'wi_cloudy_gusts', 'wi_cloudy_windy', 'wi_day_cloudy', 'wi_day_cloudy_gusts', 'wi_day_cloudy_high', 'wi_day_cloudy_windy', 'wi_day_fog', 'wi_day_hail', 'wi_day_haze', 'wi_day_light_wind', 'wi_day_lightning', 'wi_day_rain', 'wi_day_rain_mix', 'wi_day_rain_wind', 'wi_day_showers', 'wi_day_sleet', 'wi_day_sleet_storm', 'wi_day_snow', 'wi_day_snow_thunderstorm', 'wi_day_snow_wind', 'wi_day_sprinkle', 'wi_day_storm_showers', 'wi_day_sunny', 'wi_day_sunny_overcast', 'wi_day_thunderstorm', 'wi_day_windy', 'wi_degrees', 'wi_direction_down', 'wi_direction_down_left', 'wi_direction_down_right', 'wi_direction_left', 'wi_direction_right', 'wi_direction_up', 'wi_direction_up_left', 'wi_direction_up_right', 'wi_dust', 'wi_earthquake', 'wi_fahrenheit', 'wi_fire', 'wi_flood', 'wi_fog', 'wi_forecast_io_clear_day', 'wi_forecast_io_clear_night', 'wi_forecast_io_cloudy', 'wi_forecast_io_fog', 'wi_forecast_io_hail', 'wi_forecast_io_partly_cloudy_day', 'wi_forecast_io_partly_cloudy_night', 'wi_forecast_io_rain', 'wi_forecast_io_sleet', 'wi_forecast_io_snow', 'wi_forecast_io_thunderstorm', 'wi_forecast_io_tornado', 'wi_forecast_io_wind', 'wi_gale_warning', 'wi_hail', 'wi_horizon', 'wi_horizon_alt', 'wi_hot', 'wi_humidity', 'wi_hurricane', 'wi_hurricane_warning', 'wi_lightning', 'wi_lunar_eclipse', 'wi_meteor', 'wi_moon_full', 'wi_moon_new', 'wi_moon_waning_crescent_1', 'wi_moon_waning_crescent_2', 'wi_moon_waning_crescent_3', 'wi_moon_waning_crescent_4', 'wi_moon_waning_crescent_5', 'wi_moon_waning_crescent_6', 'wi_moon_waning_gibbous_1', 'wi_moon_waning_gibbous_2', 'wi_moon_waning_gibbous_3', 'wi_moon_waning_gibbous_4', 'wi_moon_waning_gibbous_5', 'wi_moon_waning_gibbous_6', 'wi_moon_waxing_cresent_1', 'wi_moon_waxing_cresent_2', 'wi_moon_waxing_cresent_3', 'wi_moon_waxing_cresent_4', 'wi_moon_waxing_cresent_5', 'wi_moon_waxing_cresent_6', 'wi_moon_waxing_gibbous_1', 'wi_moon_waxing_gibbous_2', 'wi_moon_waxing_gibbous_3', 'wi_moon_waxing_gibbous_4', 'wi_moon_waxing_gibbous_5', 'wi_moon_waxing_gibbous_6', 'wi_moonrise', 'wi_moonset', 'wi_na', 'wi_night_alt_cloudy', 'wi_night_alt_cloudy_gusts', 'wi_night_alt_cloudy_high', 'wi_night_alt_cloudy_windy', 'wi_night_alt_hail', 'wi_night_alt_lightning', 'wi_night_alt_partly_cloudy', 'wi_night_alt_rain', 'wi_night_alt_rain_mix', 'wi_night_alt_rain_wind', 'wi_night_alt_showers', 'wi_night_alt_sleet', 'wi_night_alt_sleet_storm', 'wi_night_alt_snow', 'wi_night_alt_snow_thunderstorm', 'wi_night_alt_snow_wind', 'wi_night_alt_sprinkle', 'wi_night_alt_storm_showers', 'wi_night_alt_thunderstorm', 'wi_night_clear', 'wi_night_cloudy', 'wi_night_cloudy_gusts', 'wi_night_cloudy_high', 'wi_night_cloudy_windy', 'wi_night_fog', 'wi_night_hail', 'wi_night_lightning', 'wi_night_partly_cloudy', 'wi_night_rain', 'wi_night_rain_mix', 'wi_night_rain_wind', 'wi_night_showers', 'wi_night_sleet', 'wi_night_sleet_storm', 'wi_night_snow', 'wi_night_snow_thunderstorm', 'wi_night_snow_wind', 'wi_night_sprinkle', 'wi_night_storm_showers', 'wi_night_thunderstorm', 'wi_owm_200', 'wi_owm_201', 'wi_owm_202', 'wi_owm_210', 'wi_owm_211', 'wi_owm_212', 'wi_owm_221', 'wi_owm_230', 'wi_owm_231', 'wi_owm_232', 'wi_owm_300', 'wi_owm_301', 'wi_owm_302', 'wi_owm_310', 'wi_owm_311', 'wi_owm_312', 'wi_owm_313', 'wi_owm_314', 'wi_owm_321', 'wi_owm_500', 'wi_owm_501', 'wi_owm_502', 'wi_owm_503', 'wi_owm_504', 'wi_owm_511', 'wi_owm_520', 'wi_owm_521', 'wi_owm_522', 'wi_owm_531', 'wi_owm_600', 'wi_owm_601', 'wi_owm_602', 'wi_owm_611', 'wi_owm_612', 'wi_owm_615', 'wi_owm_616', 'wi_owm_620', 'wi_owm_621', 'wi_owm_622', 'wi_owm_701', 'wi_owm_711', 'wi_owm_721', 'wi_owm_731', 'wi_owm_741', 'wi_owm_761', 'wi_owm_762', 'wi_owm_771', 'wi_owm_781', 'wi_owm_800', 'wi_owm_801', 'wi_owm_802', 'wi_owm_803', 'wi_owm_804', 'wi_owm_900', 'wi_owm_901', 'wi_owm_902', 'wi_owm_903', 'wi_owm_904', 'wi_owm_905', 'wi_owm_906', 'wi_owm_957', 'wi_owm_day_200', 'wi_owm_day_201', 'wi_owm_day_202', 'wi_owm_day_210', 'wi_owm_day_211', 'wi_owm_day_212', 'wi_owm_day_221', 'wi_owm_day_230', 'wi_owm_day_231', 'wi_owm_day_232', 'wi_owm_day_300', 'wi_owm_day_301', 'wi_owm_day_302', 'wi_owm_day_310', 'wi_owm_day_311', 'wi_owm_day_312', 'wi_owm_day_313', 'wi_owm_day_314', 'wi_owm_day_321', 'wi_owm_day_500', 'wi_owm_day_501', 'wi_owm_day_502', 'wi_owm_day_503', 'wi_owm_day_504', 'wi_owm_day_511', 'wi_owm_day_520', 'wi_owm_day_521', 'wi_owm_day_522', 'wi_owm_day_531', 'wi_owm_day_600', 'wi_owm_day_601', 'wi_owm_day_602', 'wi_owm_day_611', 'wi_owm_day_612', 'wi_owm_day_615', 'wi_owm_day_616', 'wi_owm_day_620', 'wi_owm_day_621', 'wi_owm_day_622', 'wi_owm_day_701', 'wi_owm_day_711', 'wi_owm_day_721', 'wi_owm_day_731', 'wi_owm_day_741', 'wi_owm_day_761', 'wi_owm_day_762', 'wi_owm_day_781', 'wi_owm_day_800', 'wi_owm_day_801', 'wi_owm_day_802', 'wi_owm_day_803', 'wi_owm_day_804', 'wi_owm_day_900', 'wi_owm_day_902', 'wi_owm_day_903', 'wi_owm_day_904', 'wi_owm_day_906', 'wi_owm_day_957', 'wi_owm_night_200', 'wi_owm_night_201', 'wi_owm_night_202', 'wi_owm_night_210', 'wi_owm_night_211', 'wi_owm_night_212', 'wi_owm_night_221', 'wi_owm_night_230', 'wi_owm_night_231', 'wi_owm_night_232', 'wi_owm_night_300', 'wi_owm_night_301', 'wi_owm_night_302', 'wi_owm_night_310', 'wi_owm_night_311', 'wi_owm_night_312', 'wi_owm_night_313', 'wi_owm_night_314', 'wi_owm_night_321', 'wi_owm_night_500', 'wi_owm_night_501', 'wi_owm_night_502', 'wi_owm_night_503', 'wi_owm_night_504', 'wi_owm_night_511', 'wi_owm_night_520', 'wi_owm_night_521', 'wi_owm_night_522', 'wi_owm_night_531', 'wi_owm_night_600', 'wi_owm_night_601', 'wi_owm_night_602', 'wi_owm_night_611', 'wi_owm_night_612', 'wi_owm_night_615', 'wi_owm_night_616', 'wi_owm_night_620', 'wi_owm_night_621', 'wi_owm_night_622', 'wi_owm_night_701', 'wi_owm_night_711', 'wi_owm_night_721', 'wi_owm_night_731', 'wi_owm_night_741', 'wi_owm_night_761', 'wi_owm_night_762', 'wi_owm_night_781', 'wi_owm_night_800', 'wi_owm_night_801', 'wi_owm_night_802', 'wi_owm_night_803', 'wi_owm_night_804', 'wi_owm_night_900', 'wi_owm_night_902', 'wi_owm_night_903', 'wi_owm_night_904', 'wi_owm_night_906', 'wi_owm_night_957', 'wi_rain', 'wi_rain_mix', 'wi_rain_wind', 'wi_raindrop', 'wi_raindrops', 'wi_refresh', 'wi_refresh_alt', 'wi_sandstorm', 'wi_showers', 'wi_sleet', 'wi_small_craft_advisory', 'wi_smog', 'wi_smoke', 'wi_snow', 'wi_snow_wind', 'wi_snowflake_cold', 'wi_solar_eclipse', 'wi_sprinkle', 'wi_stars', 'wi_storm_showers', 'wi_storm_warning', 'wi_strong_wind', 'wi_sunrise', 'wi_sunset', 'wi_thermometer', 'wi_thermometer_exterior', 'wi_thermometer_internal', 'wi_thunderstorm', 'wi_time_1', 'wi_time_10', 'wi_time_11', 'wi_time_12', 'wi_time_2', 'wi_time_3', 'wi_time_4', 'wi_time_5', 'wi_time_6', 'wi_time_7', 'wi_time_8', 'wi_time_9', 'wi_tornado', 'wi_train', 'wi_tsunami', 'wi_umbrella', 'wi_volcano', 'wi_wind_beaufort_0', 'wi_wind_beaufort_1', 'wi_wind_beaufort_10', 'wi_wind_beaufort_11', 'wi_wind_beaufort_12', 'wi_wind_beaufort_2', 'wi_wind_beaufort_3', 'wi_wind_beaufort_4', 'wi_wind_beaufort_5', 'wi_wind_beaufort_6', 'wi_wind_beaufort_7', 'wi_wind_beaufort_8', 'wi_wind_beaufort_9', 'wi_windy', 'wi_wmo4680_0', 'wi_wmo4680_00', 'wi_wmo4680_01', 'wi_wmo4680_02', 'wi_wmo4680_03', 'wi_wmo4680_04', 'wi_wmo4680_05', 'wi_wmo4680_1', 'wi_wmo4680_10', 'wi_wmo4680_11', 'wi_wmo4680_12', 'wi_wmo4680_18', 'wi_wmo4680_2', 'wi_wmo4680_20', 'wi_wmo4680_21', 'wi_wmo4680_22', 'wi_wmo4680_23', 'wi_wmo4680_24', 'wi_wmo4680_25', 'wi_wmo4680_26', 'wi_wmo4680_27', 'wi_wmo4680_28', 'wi_wmo4680_29', 'wi_wmo4680_3', 'wi_wmo4680_30', 'wi_wmo4680_31', 'wi_wmo4680_32', 'wi_wmo4680_33', 'wi_wmo4680_34', 'wi_wmo4680_35', 'wi_wmo4680_4', 'wi_wmo4680_40', 'wi_wmo4680_41', 'wi_wmo4680_42', 'wi_wmo4680_43', 'wi_wmo4680_44', 'wi_wmo4680_45', 'wi_wmo4680_46', 'wi_wmo4680_47', 'wi_wmo4680_48', 'wi_wmo4680_5', 'wi_wmo4680_50', 'wi_wmo4680_51', 'wi_wmo4680_52', 'wi_wmo4680_53', 'wi_wmo4680_54', 'wi_wmo4680_55', 'wi_wmo4680_56', 'wi_wmo4680_57', 'wi_wmo4680_58', 'wi_wmo4680_60', 'wi_wmo4680_61', 'wi_wmo4680_62', 'wi_wmo4680_63', 'wi_wmo4680_64', 'wi_wmo4680_65', 'wi_wmo4680_66', 'wi_wmo4680_67', 'wi_wmo4680_68', 'wi_wmo4680_70', 'wi_wmo4680_71', 'wi_wmo4680_72', 'wi_wmo4680_73', 'wi_wmo4680_74', 'wi_wmo4680_75', 'wi_wmo4680_76', 'wi_wmo4680_77', 'wi_wmo4680_78', 'wi_wmo4680_80', 'wi_wmo4680_81', 'wi_wmo4680_82', 'wi_wmo4680_83', 'wi_wmo4680_84', 'wi_wmo4680_85', 'wi_wmo4680_86', 'wi_wmo4680_87', 'wi_wmo4680_89', 'wi_wmo4680_90', 'wi_wmo4680_91', 'wi_wmo4680_92', 'wi_wmo4680_93', 'wi_wmo4680_94', 'wi_wmo4680_95', 'wi_wmo4680_96', 'wi_wmo4680_99', 'wi_yahoo_0', 'wi_yahoo_1', 'wi_yahoo_10', 'wi_yahoo_11', 'wi_yahoo_12', 'wi_yahoo_13', 'wi_yahoo_14', 'wi_yahoo_15', 'wi_yahoo_16', 'wi_yahoo_17', 'wi_yahoo_18', 'wi_yahoo_19', 'wi_yahoo_2', 'wi_yahoo_20', 'wi_yahoo_21', 'wi_yahoo_22', 'wi_yahoo_23', 'wi_yahoo_24', 'wi_yahoo_25', 'wi_yahoo_26', 'wi_yahoo_27', 'wi_yahoo_28', 'wi_yahoo_29', 'wi_yahoo_3', 'wi_yahoo_30', 'wi_yahoo_31', 'wi_yahoo_32', 'wi_yahoo_3200', 'wi_yahoo_33', 'wi_yahoo_34', 'wi_yahoo_35', 'wi_yahoo_36', 'wi_yahoo_37', 'wi_yahoo_38', 'wi_yahoo_39', 'wi_yahoo_4', 'wi_yahoo_40', 'wi_yahoo_41', 'wi_yahoo_42', 'wi_yahoo_43', 'wi_yahoo_44', 'wi_yahoo_45', 'wi_yahoo_46', 'wi_yahoo_47', 'wi_yahoo_5', 'wi_yahoo_6', 'wi_yahoo_7', 'wi_yahoo_8', 'wi_yahoo_9')

    #: All icons
    PACK_ALL = tuple(PACK_ENTYPO + PACK_FONT_AWESOME + PACK_IONICONS +
                     PACK_MATERIAL + PACK_MATERIAL_COMMUNITY + PACK_METEOCONS +
                     PACK_WEATHER)

    #: A reference to the ProxyIcon object.
    proxy = Typed(ProxyIcon)


class IconButton(Button):
    #: A reference to the ProxyIconButton object.
    proxy = Typed(ProxyIconButton)


class IconToggleButton(ToggleButton):
    #: A reference to the ProxyIconToggleButton object.
    proxy = Typed(ProxyIconToggleButton)